@@ -41,6 +41,7 @@
 #include <string.h>
 #include <time.h>
 #include <iostream>
+#include <limits>
 #include <string>
 #include <iomanip>
 
@@ -119,7 +120,7 @@ struct DisplayUI : public GenericUI {
 
 /*******************BEGIN ARCHITECTURE SECTION (part 2/2)***************/
 
-mydsp DSP;
+dsp* DSP;
 
 std::list<GUI*> GUI::fGuiList;
 ztimedmap GUI::gTimedZoneMap;
@@ -130,14 +131,17 @@ int main(int argc, char* argv[])
 {
     FAUSTFLOAT nb_samples, sample_rate, buffer_size, start_at_sample;
     
+    DSP = new mydsp();
+    
     CMDUI* interface = new CMDUI(argc, argv);
-    DSP.buildUserInterface(interface);
+    DSP->buildUserInterface(interface);
+    
+    interface->addOption("-s", &start_at_sample, 0, 0.0, 100000000.0);
     interface->addOption("-n", &nb_samples, 4096.0, 0.0, 100000000.0);
     interface->addOption("-r", &sample_rate, 44100.0, 0.0, 192000.0);
     interface->addOption("-bs", &buffer_size, kFrames, 0.0, kFrames * 16);
-    interface->addOption("-s", &start_at_sample, 0, 0.0, 100000000.0);
     
-    if (DSP.getNumInputs() > 0) {
+    if (DSP->getNumInputs() > 0) {
         cerr << "no inputs allowed " << endl;
         exit(1);
     }
@@ -146,31 +150,31 @@ int main(int argc, char* argv[])
     interface->process_one_init("-r");
     
     // init signal processor and the user interface values
-    DSP.init(sample_rate);
+    DSP->init(sample_rate);
     
     // modify the UI values according to the command line options, after init
     interface->process_init();
     
 #ifdef SOUNDFILE
     SoundUI soundinterface;
-    DSP.buildUserInterface(&soundinterface);
+    DSP->buildUserInterface(&soundinterface);
 #endif
 
     DisplayUI disp;
-    DSP.buildUserInterface(&disp);
+    DSP->buildUserInterface(&disp);
     
     // init signal processor and the user interface values
-    int nouts = DSP.getNumOutputs();
+    int nouts = DSP->getNumOutputs();
     channels chan(max(kFrames, int(buffer_size)), nouts);
     
     // skip <start> samples
     int start = int(start_at_sample);
     while (start > kFrames) {
-        DSP.compute(kFrames, nullptr, chan.buffers());
+        DSP->compute(kFrames, nullptr, chan.buffers());
         start -= kFrames;
     }
     if (start > 0) {
-        DSP.compute(start, nullptr, chan.buffers());
+        DSP->compute(start, nullptr, chan.buffers());
     }
     // end skip
     
@@ -188,7 +192,7 @@ int main(int argc, char* argv[])
     
     // print by buffer
     while (nbsamples > buffer_size) {
-        DSP.compute(buffer_size, 0, chan.buffers());
+        DSP->compute(buffer_size, 0, chan.buffers());
         for (int i = 0; i < buffer_size; i++) {
             for (int c = 0; c < nouts; c++) {
                 if (c > 0)
@@ -203,7 +207,7 @@ int main(int argc, char* argv[])
     
     // print remaining frames
     if (nbsamples) {
-        DSP.compute(nbsamples, 0, chan.buffers());
+        DSP->compute(nbsamples, 0, chan.buffers());
         for (int i = 0; i < nbsamples; i++) {
             for (int c = 0; c < nouts; c++) {
                 if (c > 0)
@@ -215,6 +219,7 @@ int main(int argc, char* argv[])
         }
     }
     
+    delete DSP;
     return 0;
 }
 
