@@ -1,26 +1,26 @@
-/************************** BEGIN one-sample-dsp.h **************************/
-/************************************************************************
- FAUST Architecture File
- Copyright (C) 2019 GRAME, Centre National de Creation Musicale
- ---------------------------------------------------------------------
- This Architecture section is free software; you can redistribute it
- and/or modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 3 of
- the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; If not, see <http://www.gnu.org/licenses/>.
- 
- EXCEPTION : As a special exception, you may create a larger work
- that contains this FAUST architecture section and distribute
- that work under terms of your choice, so long as this FAUST
- architecture section is not modified.
- ************************************************************************/
+/************************** BEGIN one-sample-dsp.h ************************
+FAUST Architecture File
+Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
+---------------------------------------------------------------------
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation; either version 2.1 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+EXCEPTION : As a special exception, you may create a larger work
+that contains this FAUST architecture section and distribute
+that work under terms of your choice, so long as this FAUST
+architecture section is not modified.
+***************************************************************************/
 
 #ifndef __one_sample_dsp__
 #define __one_sample_dsp__
@@ -28,7 +28,7 @@
 #include <assert.h>
 #include "faust/dsp/dsp.h"
 
-class one_sample_dsp : public dsp {
+class FAUST_API one_sample_dsp : public dsp {
   
     protected:
         
@@ -58,9 +58,9 @@ class one_sample_dsp : public dsp {
         one_sample_dsp():fInputs(nullptr), fOutputs(nullptr), iControl(nullptr), fControl(nullptr), fDelete(true)
         {}
         
-        one_sample_dsp(int* icontrol, FAUSTFLOAT* fcontrol)
+        one_sample_dsp(int* iControl, FAUSTFLOAT* fControl)
         :fInputs(nullptr), fOutputs(nullptr),
-        iControl(icontrol), fControl(fcontrol), fDelete(false)
+        iControl(iControl), fControl(fControl), fDelete(false)
         {}
         
         virtual ~one_sample_dsp()
@@ -159,8 +159,10 @@ class one_sample_dsp : public dsp {
     
 };
 
+// To be used with -os1 and -os2 mode
+
 template <typename REAL>
-class one_sample_dsp_real : public dsp {
+class FAUST_API one_sample_dsp_real : public dsp {
     
     protected:
     
@@ -198,10 +200,10 @@ class one_sample_dsp_real : public dsp {
         iZone(nullptr), fZone(nullptr), fDelete(true)
         {}
     
-        one_sample_dsp_real(int* icontrol, FAUSTFLOAT* fcontrol, int* izone, REAL* fzone)
+        one_sample_dsp_real(int* iControl, FAUSTFLOAT* fControl, int* iZone, REAL* fZone)
         :fInputs(nullptr), fOutputs(nullptr),
-        iControl(icontrol), fControl(fcontrol),
-        iZone(izone), fZone(fzone), fDelete(false)
+        iControl(iControl), fControl(fControl),
+        iZone(iZone), fZone(fZone), fDelete(false)
         {}
         
         virtual ~one_sample_dsp_real()
@@ -358,5 +360,117 @@ class one_sample_dsp_real : public dsp {
     
 };
 
+// To be used with -os3 mode
+
+template <typename REAL>
+class FAUST_API one_sample_dsp_real1 : public dsp {
+    
+    protected:
+        
+        FAUSTFLOAT* fInputs;
+        FAUSTFLOAT* fOutputs;
+            
+    public:
+        
+        one_sample_dsp_real1():fInputs(nullptr), fOutputs(nullptr)
+        {}
+        
+        virtual ~one_sample_dsp_real1()
+        {
+            delete [] fInputs;
+            delete [] fOutputs;
+        }
+        
+        /**
+         * Return the number of 'int' typed values necessary to compute the internal DSP state
+         *
+         * @return the number of 'int' typed values.
+         */
+        virtual int getNumIntControls() = 0;
+        
+        /**
+         * Return the number of 'float, double or quad' typed values necessary to compute the DSP control state
+         *
+         * @return the number of 'float, double or quad' typed values.
+         */
+        virtual int getNumRealControls() = 0;
+        
+        /**
+         * Return the size on 'float, double or quad' typed values necessary to compute the DSP state
+         *
+         * @return the number of 'float, double or quad' typed values.
+         */
+        virtual int getiZoneSize() = 0;
+        
+        /**
+         * Return the size on 'int' typed values necessary to compute the DSP state
+         *
+         * @return the number of 'int' typed values.
+         */
+        virtual int getfZoneSize() = 0;
+        
+        /**
+         * Update the DSP control state.
+         */
+        virtual void control() = 0;
+        
+        /**
+         * Compute one sample.
+         *
+         * @param inputs - the input audio buffers as an array of getNumInputs FAUSTFLOAT samples (either float, double or quad)
+         * @param outputs - the output audio buffers as an array of getNumOutputs FAUSTFLOAT samples (either float, double or quad)
+         */
+        virtual void compute(FAUSTFLOAT* inputs, FAUSTFLOAT* outputs) = 0;
+        
+        // The standard 'compute' expressed using the control/compute (one sample) model
+        virtual void compute(int count, FAUSTFLOAT** inputs_aux, FAUSTFLOAT** outputs_aux)
+        {
+            // TODO : not RT safe
+            if (!fInputs) {
+                fInputs = new FAUSTFLOAT[getNumInputs() * 4096];
+                fOutputs = new FAUSTFLOAT[getNumOutputs() * 4096];
+            }
+            
+            // Control
+            control();
+            
+            // Compute
+            int num_inputs = getNumInputs();
+            int num_outputs = getNumOutputs();
+            
+            FAUSTFLOAT* inputs_ptr = &fInputs[0];
+            FAUSTFLOAT* outputs_ptr = &fOutputs[0];
+            
+            for (int frame = 0; frame < count; frame++) {
+                for (int chan = 0; chan < num_inputs; chan++) {
+                    inputs_ptr[chan] = inputs_aux[chan][frame];
+                }
+                inputs_ptr += num_inputs;
+            }
+            
+            inputs_ptr = &fInputs[0];
+            for (int frame = 0; frame < count; frame++) {
+                // One sample compute
+                compute(inputs_ptr, outputs_ptr);
+                inputs_ptr += num_inputs;
+                outputs_ptr += num_outputs;
+            }
+            
+            outputs_ptr = &fOutputs[0];
+            for (int frame = 0; frame < count; frame++) {
+                for (int chan = 0; chan < num_outputs; chan++) {
+                    outputs_aux[chan][frame] = outputs_ptr[chan];
+                }
+                outputs_ptr += num_outputs;
+            }
+        }
+        
+        virtual void compute(double date_usec, int count, FAUSTFLOAT** inputs, FAUSTFLOAT** outputs)
+        {
+            compute(count, inputs, outputs);
+        }
+  
+};
+
 #endif
 /************************** END one-sample-dsp.h **************************/
