@@ -1,26 +1,26 @@
-/************************** BEGIN wasm-dsp-imp.h **************************/
-/************************************************************************
- FAUST Architecture File
- Copyright (C) 2020 GRAME, Centre National de Creation Musicale
- ---------------------------------------------------------------------
- This Architecture section is free software; you can redistribute it
- and/or modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 3 of
- the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; If not, see <http://www.gnu.org/licenses/>.
- 
- EXCEPTION : As a special exception, you may create a larger work
- that contains this FAUST architecture section and distribute
- that work under terms of your choice, so long as this FAUST
- architecture section is not modified.
- ************************************************************************/
+/************************** BEGIN wasm-dsp-imp.h **************************
+FAUST Architecture File
+Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
+---------------------------------------------------------------------
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation; either version 2.1 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+EXCEPTION : As a special exception, you may create a larger work
+that contains this FAUST architecture section and distribute
+that work under terms of your choice, so long as this FAUST
+architecture section is not modified.
+***************************************************************************/
 
 #ifndef wasm_dsp_imp_H
 #define wasm_dsp_imp_H
@@ -32,10 +32,11 @@
 #include "faust/dsp/dsp.h"
 #include "faust/gui/meta.h"
 #include "faust/gui/JSONUIDecoder.h"
+#include "faust/export.h"
 
 // Generic wasm_dsp_factory class that keeps the JSON decoder.
 
-class wasm_dsp_factory_imp : public dsp_factory {
+class LIFAUST_API wasm_dsp_factory_imp : public dsp_factory {
     
     public:
     
@@ -68,7 +69,7 @@ class wasm_dsp_factory_imp : public dsp_factory {
 
 // Generic wasm_dsp class that creates and use the JSON decoder, and manage DSP and audio memory.
 
-class wasm_dsp_imp : public dsp {
+class LIFAUST_API wasm_dsp_imp : public dsp {
     
     protected:
     
