@@ -1,26 +1,26 @@
-/************************** BEGIN SoundUI.h **************************/
-/************************************************************************
+/************************** BEGIN SoundUI.h **************************
  FAUST Architecture File
- Copyright (C) 2018 GRAME, Centre National de Creation Musicale
+ Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
  ---------------------------------------------------------------------
- This Architecture section is free software; you can redistribute it
- and/or modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 3 of
- the License, or (at your option) any later version.
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU Lesser General Public License as published by
+ the Free Software Foundation; either version 2.1 of the License, or
+ (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU Lesser General Public License for more details.
  
- You should have received a copy of the GNU General Public License
- along with this program; If not, see <http://www.gnu.org/licenses/>.
+ You should have received a copy of the GNU Lesser General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  
  EXCEPTION : As a special exception, you may create a larger work
  that contains this FAUST architecture section and distribute
  that work under terms of your choice, so long as this FAUST
  architecture section is not modified.
- ************************************************************************/
+ ********************************************************************/
  
 #ifndef __SoundUI_H__
 #define __SoundUI_H__
@@ -29,6 +29,7 @@
 #include <vector>
 #include <string>
 #include <iostream>
+#include <memory>
 
 #include "faust/gui/SimpleParser.h"
 #include "faust/gui/DecoratorUI.h"
@@ -66,9 +67,12 @@ class SoundUI : public SoundUIInterface
 		
     protected:
     
-        std::vector<std::string> fSoundfileDir;             // The soundfile directories
-        std::map<std::string, Soundfile*> fSoundfileMap;    // Map to share loaded soundfiles
-        SoundfileReader* fSoundReader;
+        // The soundfile directories
+        std::vector<std::string> fSoundfileDir;
+        // Map to share loaded soundfiles
+        std::map<std::string, std::shared_ptr<Soundfile>> fSoundfileMap;
+        // The soundfile reader
+        std::shared_ptr<SoundfileReader> fSoundReader;
         bool fIsDouble;
 
      public:
@@ -79,13 +83,17 @@ class SoundUI : public SoundUIInterface
          * @param sound_directory - the base directory to look for files, which paths will be relative to this one
          * @param sample_rate - the audio driver SR which may be different from the file SR, to possibly resample files
          * @param reader - an alternative soundfile reader
+         * @param is_double - whether Faust code has been compiled in -double mode and soundfile buffers have to be in double
          *
          * @return the soundfile loader.
          */
         SoundUI(const std::string& sound_directory = "", int sample_rate = -1, SoundfileReader* reader = nullptr, bool is_double = false)
         {
             fSoundfileDir.push_back(sound_directory);
-            fSoundReader = (reader) ? reader : &gReader;
+            fSoundReader = (reader)
+                ? std::shared_ptr<SoundfileReader>(reader)
+                // the static gReader should not be deleted, so use an empty destructor
+                : std::shared_ptr<SoundfileReader>(std::shared_ptr<SoundfileReader>{}, &gReader);
             fSoundReader->setSampleRate(sample_rate);
             fIsDouble = is_double;
             if (!defaultsound) defaultsound = gReader.createSoundfile(path_name_list, MAX_CHAN, is_double);
@@ -97,25 +105,24 @@ class SoundUI : public SoundUIInterface
          * @param sound_directories - a vector of base directories to look for files, which paths will be relative to these ones
          * @param sample_rate - the audio driver SR which may be different from the file SR, to possibly resample files
          * @param reader - an alternative soundfile reader
+         * @param is_double - whether Faust code has been compiled in -double mode and soundfile buffers have to be in double
          *
          * @return the soundfile loader.
          */
         SoundUI(const std::vector<std::string>& sound_directories, int sample_rate = -1, SoundfileReader* reader = nullptr, bool is_double = false)
         :fSoundfileDir(sound_directories)
         {
-            fSoundReader = (reader) ? reader : &gReader;
+            fSoundReader = (reader)
+                ? std::shared_ptr<SoundfileReader>(reader)
+                // the static gReader should not be deleted, so use an empty destructor
+                : std::shared_ptr<SoundfileReader>(std::shared_ptr<SoundfileReader>{}, &gReader);
             fSoundReader->setSampleRate(sample_rate);
             fIsDouble = is_double;
             if (!defaultsound) defaultsound = gReader.createSoundfile(path_name_list, MAX_CHAN, is_double);
         }
     
         virtual ~SoundUI()
-        {   
-            // Delete all soundfiles
-            for (const auto& it : fSoundfileMap) {
-                delete it.second;
-            }
-        }
+        {}
 
         // -- soundfiles
         virtual void addSoundfile(const char* label, const char* url, Soundfile** sf_zone)
@@ -135,7 +142,7 @@ class SoundUI : public SoundUIInterface
                 // Read them and create the Soundfile
                 Soundfile* sound_file = fSoundReader->createSoundfile(path_name_list, MAX_CHAN, fIsDouble);
                 if (sound_file) {
-                    fSoundfileMap[saved_url_real] = sound_file;
+                    fSoundfileMap[saved_url_real] = std::shared_ptr<Soundfile>(sound_file);
                 } else {
                     // If failure, use 'defaultsound'
                     std::cerr << "addSoundfile : soundfile for " << saved_url << " cannot be created !" << std::endl;
@@ -144,8 +151,8 @@ class SoundUI : public SoundUIInterface
                 }
             }
             
-            // Get the soundfile
-            *sf_zone = fSoundfileMap[saved_url_real];
+            // Get the soundfile pointer
+            *sf_zone = fSoundfileMap[saved_url_real].get();
         }
     
         /**
