@@ -1,37 +1,35 @@
-/************************** BEGIN mspUI.h **************************/
-/************************************************************************
+/************************** BEGIN mspUI.h **************************
  FAUST Architecture File
- Copyright (C) 2018 GRAME, Centre National de Creation Musicale
+ Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
  ---------------------------------------------------------------------
- This Architecture section is free software; you can redistribute it
- and/or modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 3 of
- the License, or (at your option) any later version.
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU Lesser General Public License as published by
+ the Free Software Foundation; either version 2.1 of the License, or
+ (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU Lesser General Public License for more details.
  
- You should have received a copy of the GNU General Public License
- along with this program; If not, see <http://www.gnu.org/licenses/>.
+ You should have received a copy of the GNU Lesser General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  
  EXCEPTION : As a special exception, you may create a larger work
  that contains this FAUST architecture section and distribute
  that work under terms of your choice, so long as this FAUST
  architecture section is not modified.
- ************************************************************************/
-//
-//  mspUI.h for static Max/MSP externals and faustgen~
-//
-//  Created by Martin Di Rollo on 18/04/12.
-//  Copyright (c) 2012-2019 Grame. All rights reserved.
-//
+ 
+ mspUI.h for static Max/MSP externals and faustgen~
+ Created by Martin Di Rollo on 18/04/12.
+ ********************************************************************/
 
 #ifndef _mspUI_h
 #define _mspUI_h
 
 #include <math.h>
+#include <assert.h>
 #include <string>
 #include <map>
 
@@ -226,8 +224,10 @@ class mspUI : public UI, public PathBuilder
     private:
         
         map<string, mspUIObject*> fInputLabelTable;      // Input table using labels
+        map<string, mspUIObject*> fInputShortnameTable;  // Input table using shortnames
         map<string, mspUIObject*> fInputPathTable;       // Input table using paths
         map<string, mspUIObject*> fOutputLabelTable;     // Table containing bargraph with labels
+        map<string, mspUIObject*> fOutputShortnameTable; // Table containing bargraph with shortnames
         map<string, mspUIObject*> fOutputPathTable;      // Table containing bargraph with paths
         
         map<const char*, const char*> fDeclareTable;
@@ -240,10 +240,9 @@ class mspUI : public UI, public PathBuilder
         {
             map<const char*, const char*>::reverse_iterator it;
             if (fDeclareTable.size() > 0) {
-                unsigned int i = 0;
                 string res = string(label);
                 char sep = '[';
-                for (it = fDeclareTable.rbegin(); it != fDeclareTable.rend(); it++, i++) {
+                for (it = fDeclareTable.rbegin(); it != fDeclareTable.rend(); it++) {
                     res = res + sep + (*it).first + ":" + (*it).second;
                     sep = ',';
                 }
@@ -259,7 +258,9 @@ class mspUI : public UI, public PathBuilder
         {
             mspUIObject* obj = new mspSlider(createLabel(label), zone, init, min, max, step);
             fInputLabelTable[string(label)] = obj;
-            fInputPathTable[buildPath(label)] = obj;
+            string path = buildPath(label);
+            fInputPathTable[path] = obj;
+            fFullPaths.push_back(path);
             fDeclareTable.clear();
         }
     
@@ -267,7 +268,9 @@ class mspUI : public UI, public PathBuilder
         {
             mspUIObject* obj = new mspBargraph(createLabel(label), zone, min, max);
             fOutputLabelTable[string(label)] = obj;
-            fOutputPathTable[buildPath(label)] = obj;
+            string path = buildPath(label);
+            fOutputPathTable[path] = obj;
+            fFullPaths.push_back(path);
             fDeclareTable.clear();
         }
     
@@ -292,14 +295,18 @@ class mspUI : public UI, public PathBuilder
         {
             mspUIObject* obj = new mspButton(createLabel(label), zone);
             fInputLabelTable[string(label)] = obj;
-            fInputPathTable[buildPath(label)] = obj;
+            string path = buildPath(label);
+            fInputPathTable[path] = obj;
+            fFullPaths.push_back(path);
         }
         
         void addCheckButton(const char* label, FAUSTFLOAT* zone)
         {
             mspUIObject* obj = new mspCheckButton(createLabel(label), zone);
             fInputLabelTable[string(label)] = obj;
-            fInputPathTable[buildPath(label)] = obj;
+            string path = buildPath(label);
+            fInputPathTable[path] = obj;
+            fFullPaths.push_back(path);
         }
         
         void addVerticalSlider(const char* label, FAUSTFLOAT* zone, FAUSTFLOAT init, FAUSTFLOAT min, FAUSTFLOAT max, FAUSTFLOAT step)
@@ -332,7 +339,24 @@ class mspUI : public UI, public PathBuilder
         void openTabBox(const char* label) { pushLabel(label); fDeclareTable.clear(); }
         void openHorizontalBox(const char* label) { pushLabel(label); fDeclareTable.clear(); }
         void openVerticalBox(const char* label) { pushLabel(label); fDeclareTable.clear(); }
-        void closeBox() { popLabel(); fDeclareTable.clear(); }
+        void closeBox()
+        {
+            fDeclareTable.clear();
+            if (popLabel()) {
+                // Shortnames can be computed when all fullnames are known
+                computeShortNames();
+                // Fill 'shortname' map
+                for (const auto& path : fFullPaths) {
+                    if (fInputPathTable.count(path)) {
+                        fInputShortnameTable[fFull2Short[path]] = fInputPathTable[path];
+                    } else if (fOutputPathTable.count(path)) {
+                        fOutputShortnameTable[fFull2Short[path]] = fOutputPathTable[path];
+                    } else {
+                        assert(false);
+                    }
+                }
+             }
+        }
         
         virtual void declare(FAUSTFLOAT* zone, const char* key, const char* val)
         {
@@ -364,17 +388,17 @@ class mspUI : public UI, public PathBuilder
         
         bool isValue(const string& name)
         {
-            return (fInputLabelTable.count(name) || fInputPathTable.count(name));
-        }
-    
-        bool isOutputValue(const string& name)
-        {
-            return fOutputPathTable.count(name);
+            return (isOutputValue(name) || isInputValue(name));
         }
     
         bool isInputValue(const string& name)
         {
-            return fInputPathTable.count(name);
+            return fInputLabelTable.count(name) || fInputShortnameTable.count(name) || fInputPathTable.count(name);
+        }
+    
+        bool isOutputValue(const string& name)
+        {
+            return fOutputLabelTable.count(name) || fOutputShortnameTable.count(name) || fOutputPathTable.count(name);
         }
     
         bool setValue(const string& name, FAUSTFLOAT val)
@@ -382,6 +406,9 @@ class mspUI : public UI, public PathBuilder
             if (fInputLabelTable.count(name)) {
                 fInputLabelTable[name]->setValue(val);
                 return true;
+            } else if (fInputShortnameTable.count(name)) {
+                fInputShortnameTable[name]->setValue(val);
+                return true;
             } else if (fInputPathTable.count(name)) {
                 fInputPathTable[name]->setValue(val);
                 return true;
@@ -416,26 +443,30 @@ class mspUI : public UI, public PathBuilder
                 delete it.second;
             }
             fInputLabelTable.clear();
+            fInputShortnameTable.clear();
             fInputPathTable.clear();
             
             for (const auto& it : fOutputLabelTable) {
                 delete it.second;
             }
             fOutputLabelTable.clear();
+            fOutputShortnameTable.clear();
             fOutputPathTable.clear();
         }
         
         void displayControls()
         {
-            post("------- Range and path ----------");
+            post("------- Range, shortname and path ----------");
             for (const auto& it : fInputPathTable) {
                 char param[STR_SIZE];
                 it.second->toString(param);
                 post(param);
+                string shortname = "Shortname: " + fFull2Short[it.first];
+                post(shortname.c_str());
                 string path = "Complete path: " + it.first;
                 post(path.c_str());
             }
-            post("---------------------------------");
+            post("---------------------------------------------");
         }
     
         static bool checkDigit(const string& name)
