@@ -1,18 +1,25 @@
 /************************************************************************
- ************************************************************************
  FAUST Architecture File
- Copyright (C) 2003-2012 GRAME, Centre National de Creation Musicale
+ Copyright (C) 2003-2019 GRAME, Centre National de Creation Musicale
  ---------------------------------------------------------------------
+ This Architecture section is free software; you can redistribute it
+ and/or modify it under the terms of the GNU General Public License
+ as published by the Free Software Foundation; either version 3 of
+ the License, or (at your option) any later version.
  
- This is sample code. This file is provided as an example of minimal
- FAUST architecture file. Redistribution and use in source and binary
- forms, with or without modification, in part or in full are permitted.
- In particular you can create a derived work of this FAUST architecture
- and distribute that work under terms of your choice.
- 
- This sample code is distributed in the hope that it will be useful,
+ This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+ 
+ You should have received a copy of the GNU General Public License
+ along with this program; If not, see <http://www.gnu.org/licenses/>.
+ 
+ EXCEPTION : As a special exception, you may create a larger work
+ that contains this FAUST architecture section and distribute
+ that work under terms of your choice, so long as this FAUST
+ architecture section is not modified.
+ 
  ************************************************************************
  ************************************************************************/
 
@@ -57,6 +64,7 @@ ztimedmap GUI::gTimedZoneMap;
 audio* audio_device = NULL;
 CocoaUI* uiinterface = NULL;
 FUI* finterface = NULL;
+SaveLabelUI* saveinterface = NULL;
 
 #if SOUNDFILE
 SoundUI* soundinterface = NULL;
@@ -167,6 +175,8 @@ float uiBox::gDummy = 0;
     
     uiinterface = new CocoaUI([UIApplication sharedApplication].keyWindow, self, &metadata, DSP);
     finterface = new FUI();
+    saveinterface = new SaveLabelUI();
+    
 #if MIDICTRL
     midi_handler = new rt_midi(_name);
     midiinterface = new MidiUI(midi_handler);
@@ -193,6 +203,7 @@ float uiBox::gDummy = 0;
     
     DSP->buildUserInterface(uiinterface);
     DSP->buildUserInterface(finterface);
+    DSP->buildUserInterface(saveinterface);
     
 #if SOUNDFILE
     // Use bundle path
@@ -362,6 +373,7 @@ error:
     
     delete uiinterface;
     delete finterface;
+    delete saveinterface;
     
 #if SOUNDFILE
     delete soundinterface;
@@ -874,6 +886,9 @@ static inline const char* transmit_value(int num)
 - (void)setOSCParameters:(int)transmit output:(NSString*)outputIPText inputport:(NSString*)inputPortText outputport:(NSString*)outputPortText;
 {
 #if OSCCTRL
+    // Save current stare;
+    saveinterface->save();
+    
     delete oscinterface;
     const char* argv[9];
     argv[0] = (char*)_name;
@@ -885,14 +900,22 @@ static inline const char* transmit_value(int num)
     argv[6] = [inputPortText cStringUsingEncoding:[NSString defaultCStringEncoding]];
     argv[7] = "-outport";
     argv[8] = [outputPortText cStringUsingEncoding:[NSString defaultCStringEncoding]];
+    
     /*
     // Deactivated for now (sometimes crashing)
     argv[9] = "-bundle";
     argv[10] = "1";
     */
+    
+    // Start OSC interface
     oscinterface = new OSCUI(_name, 9, (char**)argv);
     DSP->buildUserInterface(oscinterface);
     audio_device->addControlCallback(osc_compute_callback, self);
+    
+    // Load current controller state
+    DSP->buildUserInterface(saveinterface);
+    
+    // Start OSC
     oscinterface->run();
 #endif
 }
