@@ -37,6 +37,7 @@
 #include <list>
 
 #include "faust/dsp/timed-dsp.h"
+#include "faust/dsp/one-sample-dsp.h"
 #include "faust/gui/FUI.h"
 #include "faust/gui/PrintUI.h"
 #include "faust/misc.h"
@@ -114,6 +115,11 @@ int main(int argc, char* argv[])
     int nvoices = 0;
     bool control = true;
     
+    if (isopt(argv, "-h")) {
+        cout << argv[0] << " [--nvoices <val>] [--control <0/1>] [--group <0/1>]\n";
+        exit(1);
+    }
+    
     mydsp* tmp_dsp = new mydsp();
     MidiMeta::analyse(tmp_dsp, midi_sync, nvoices);
     delete tmp_dsp;
@@ -121,11 +127,6 @@ int main(int argc, char* argv[])
     snprintf(name, 256, "%s", basename(argv[0]));
     snprintf(rcfilename, 256, "%s/.%src", home, name);
     
-    if (isopt(argv, "-h")) {
-        cout << "prog [--nvoices <val>] [--control <0/1>] [--group <0/1>]\n";
-        exit(1);
-    }
-    
 #ifdef POLY2
     nvoices = lopt(argv, "--nvoices", nvoices);
     control = lopt(argv, "--control", control);
@@ -201,11 +202,13 @@ int main(int argc, char* argv[])
     
 #ifdef MIDICTRL
     jackaudio_midi audio;
-    audio.init(name, DSP);
 #else
     jackaudio audio;
-    audio.init(name, DSP);
 #endif
+    if (!audio.init(name, DSP)) {
+        cerr << "Unable to init audio" << endl;
+        exit(1);
+    }
     
 // After audio init to get SR
 #if SOUNDFILE
@@ -227,7 +230,10 @@ int main(int argc, char* argv[])
     cout << "MIDI is on" << endl;
 #endif
     
-    audio.start();
+    if (!audio.start()) {
+        cerr << "Unable to start audio" << endl;
+        exit(1);
+    }
     
     cout << "ins " << audio.getNumInputs() << endl;
     cout << "outs " << audio.getNumOutputs() << endl;
