@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -32,7 +32,7 @@
  * \version 1.0
  * \date 2003
  * \brief A simple type system for block diagram expressions.
- *  The type of a block diagram is defined by a number of inputs and outputs.
+ * The type of a block diagram is defined by a number of inputs and outputs.
  */
 
 #include <stdio.h>
@@ -55,7 +55,7 @@ static bool infereBoxType(Tree box, int* inum, int* onum);
  * \return true if type is defined, false if undefined
  */
 
-bool getBoxType(Tree box, int* inum, int* onum)
+LIBFAUST_API bool getBoxType(Tree box, int* inum, int* onum)
 {
     Tree t;
     if (getProperty(box, gGlobal->BOXTYPEPROP, t)) {
@@ -80,7 +80,7 @@ bool getBoxType(Tree box, int* inum, int* onum)
 
 /**
  * Produces "1 output" or "n outputs" according to n
- * \param n: the number of outputs
+ * \param n the number of outputs
  * \return the corresponding string
  */
 
@@ -93,7 +93,7 @@ static string outputs(int n)
 
 /**
  * Produces "1 input" or "n inputs" according to n
- * \param n: the number of inputs
+ * \param n the number of inputs
  * \return the corresponding string
  */
 
@@ -106,13 +106,13 @@ static string inputs(int n)
 
 /**
  * Compute a parametric type error message
- * \param a: the LHS expression
- * \param b: the RHS expression
- * \param o: the number of outputs of a
- * \param i: the number of inputs of b
- * \param opcode = the composition operation sign
- * \param opname = the full name of the composition operation
- * \param msg  = indication for the user (ie " must be equal to ")
+ * \param a the LHS expression
+ * \param b the RHS expression
+ * \param o the number of outputs of a
+ * \param i the number of inputs of b
+ * \param opcode the composition operation sign
+ * \param opname the full name of the composition operation
+ * \param msg indication for the user (ie " must be equal to ")
  * \return the error message as a string
  */
 
@@ -134,12 +134,12 @@ static string computeTypeErrorMessage(Tree a, Tree b, int o, int i, const string
 
 /**
  * Compute a parametric type error message for recursions
- * \param a: the LHS expression
- * \param b: the RHS expression
- * \param u: the number of inputs of a
- * \param v: the number of outputs of a
- * \param x: the number of inputs of b
- * \param y: the number of outputs of b
+ * \param a the LHS expression
+ * \param b the RHS expression
+ * \param u the number of inputs of a
+ * \param v the number of outputs of a
+ * \param x the number of inputs of b
+ * \param y the number of outputs of b
  * \return the error message as a string
  */
 
@@ -344,7 +344,7 @@ static bool infereBoxType(Tree t, int* inum, int* onum)
         return getBoxType(a, inum, onum);
 
     } else if (isBoxRoute(t, ins, outs, lroutes)) {
-        return isBoxInt(ins, inum) & isBoxInt(outs, onum);
+        return isBoxInt(ins, inum) && isBoxInt(outs, onum);
 
     } else {
         stringstream error;
