@@ -4,16 +4,16 @@
  Copyright (C) 2009 GRAME, Centre National de Creation Musicale
  ---------------------------------------------------------------------
  This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ it under the terms of the GNU Lesser General Public License as published by
+ the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Lesser General Public License for more details.
 
- You should have received a copy of the GNU General Public License
+ You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -784,7 +784,7 @@ static void getBoxInputsAndOutputs(const Tree t, int& numInputs, int& numOutputs
 {
     if (!getBoxType(t, &numInputs, &numOutputs)) {
         stringstream error;
-        error << "ERROR during the evaluation of t : " << boxpp(t) << endl;
+        error << "ERROR : during the evaluation of t : " << boxpp(t) << endl;
         throw faustexception(error.str());
     }
     // cerr << "Documentator : " << numInputs <<" inputs and " << numOutputs <<" outputs for box : " << boxpp(t) <<
@@ -825,7 +825,7 @@ static void printDocDgm(const Tree expr, const char* svgTopDir, ostream& docout,
     Tree docdgm = evaldocexpr(expr, gGlobal->gExpandedDefList);
     if (gGlobal->gErrorCount > 0) {
         stringstream error;
-        error << "Total of " << gGlobal->gErrorCount
+        error << "ERROR : total of " << gGlobal->gErrorCount
               << " errors during evaluation of : diagram docdgm = " << boxpp(docdgm) << ";\n";
         throw faustexception(error.str());
     }
