@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -199,12 +199,7 @@ string DocCompiler::generateCode(Tree sig, int priority)
     } else if (isSigPrefix(sig, x, y)) {
         printGCCall(sig, "generatePrefix");
         return generatePrefix(sig, x, y, priority);
-    } else if (isSigIota(sig, x)) {
-        printGCCall(sig, "generateIota");
-        return generateIota(sig, x);
-    }
-
-    else if (isSigBinOp(sig, &i, x, y)) {
+    } else if (isSigBinOp(sig, &i, x, y)) {
         printGCCall(sig, "generateBinOp");
         return generateBinOp(sig, i, x, y, priority);
     } else if (isSigFFun(sig, ff, largs)) {
@@ -283,12 +278,12 @@ string DocCompiler::generateCode(Tree sig, int priority)
     }
 
     else {
-        stringstream error;
-        error << "ERROR in d signal, unrecognized signal : " << *sig << endl;
-        throw faustexception(error.str());
+        cerr << "ERROR : unrecognized signal : " << *sig << endl;
+        faustassert(false);
     }
-    faustassert(0);
-    return "error in generate code";
+    faustassert(false);
+    // Never reached
+    return "ERROR : in generate code";
 }
 
 /**
@@ -988,24 +983,7 @@ string DocCompiler::generatePrefix(Tree sig, Tree x, Tree e, int priority)
     return generateCacheCode(sig, subst("$0(t)", var));
 }
 
-/*****************************************************************************
-                               IOTA(n)
-*****************************************************************************/
-
-/**
- * Generate a "iota" time function, n-cyclical.
- */
-string DocCompiler::generateIota(Tree sig, Tree n)
-{
-    int size;
-    if (!isSigInt(n, &size)) {
-        throw faustexception("ERROR in generateIota\n");
-    }
-    // cout << "iota !" << endl;
-    return subst(" t \\bmod{$0} ", docT(size));
-}
-
-// a revoir en utilisant la lecture de table et en partageant la construction de la paire de valeurs
+// to be reviewed using table reading and sharing the construction of the value pair
 
 /**
  * Generate a select2 code
@@ -1032,33 +1010,6 @@ string DocCompiler::generateSelect2(Tree sig, Tree sel, Tree s1, Tree s2, int pr
     return subst("$0(t)", var);
 }
 
-/**
- * Generate a select3 code
- */
-string DocCompiler::generateSelect3(Tree sig, Tree sel, Tree s1, Tree s2, Tree s3, int priority)
-{
-    string var    = getFreshID("q");
-    string expsel = CS(sel, 0);
-    string exps1  = CS(s1, 0);
-    string exps2  = CS(s2, 0);
-    string exps3  = CS(s3, 0);
-
-    string ltqSelDef;
-    ltqSelDef += subst("$0(t) = \n", var);
-    ltqSelDef += "\\left\\{\\begin{array}{ll}\n";
-    ltqSelDef += subst("$0 & \\mbox{if \\,} $1 = 0\\\\\n", generateVariableStore(s1, exps1), expsel);
-    ltqSelDef += subst("$0 & \\mbox{if \\,} $1 = 1\\\\\n", generateVariableStore(s2, exps2), expsel);
-    ltqSelDef += subst("$0 & \\mbox{if \\,} $1 = 2\n", generateVariableStore(s3, exps3), expsel);
-    ltqSelDef += "\\end{array}\\right.";
-
-    fLateq->addSelectSigFormula(ltqSelDef);
-    gGlobal->gDocNoticeFlagMap["selectionsigs"] = true;
-
-    // return generateCacheCode(sig, subst("$0(t)", var));
-    setVectorNameProperty(sig, var);
-    return subst("$0(t)", var);
-}
-
 /**
  * retrieve the type annotation of sig
  * @param sig the signal we want to know the type
