@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -29,6 +29,7 @@
 #include "recursivness.hh"
 #include "text_instructions.hh"
 #include "type_manager.hh"
+#include "struct_manager.hh"
 
 using namespace std;
 
@@ -39,7 +40,7 @@ void CodeContainer::initialize(int numInputs, int numOutputs)
 }
 
 CodeContainer::CodeContainer()
-    : fParentContainer(0),
+    : fParentContainer(nullptr),
       fNumInputs(-1),
       fNumOutputs(-1),
       fNumActives(0),
@@ -197,15 +198,12 @@ void CodeContainer::printLibrary(ostream& fout)
  */
 void CodeContainer::printIncludeFile(ostream& fout)
 {
-    set<string>           S;
-    set<string>::iterator f;
-
+    set<string> S;
     collectIncludeFile(S);
-    for (f = S.begin(); f != S.end(); f++) {
-        string inc = *f;
+    for (const auto& inc : S) {
         // Only print non-empty include (inc is actually quoted)
         if (inc.size() > 2) {
-            fout << "#include " << *f << "\n";
+            fout << "#include " << inc << "\n";
         }
     }
 }
@@ -272,9 +270,9 @@ void CodeContainer::computeForwardDAG(lclgraph dag, int& loop_count, vector<int>
 }
 
 ValueInst* CodeContainer::pushFunction(const string& name, Typed::VarType result, vector<Typed::VarType>& types,
-                                       const list<ValueInst*>& args)
+                                       const Values& args)
 {
-    list<NamedTyped*> named_args;
+    Names named_args;
     for (size_t i = 0; i < types.size(); i++) {
         named_args.push_back(InstBuilder::genNamedTyped("dummy" + to_string(i), InstBuilder::genBasicTyped(types[i])));
     }
@@ -303,12 +301,13 @@ void CodeContainer::sortDeepFirstDAG(CodeLoop* l, set<CodeLoop*>& visited, list<
 
 // Functions are coded with a "class" prefix, so to stay separated in "gGlobalTable"
 void CodeContainer::produceInfoFunctions(int tabs, const string& classname, const string& obj, bool ismethod,
-                                         bool isvirtual, TextInstVisitor* producer)
+                                         FunTyped::FunAttribute funtype, TextInstVisitor* producer,
+                                         const string& in_fun, const string& out_fun)
 {
     // Input/Output method
     producer->Tab(tabs);
-    generateGetInputs(subst("getNumInputs$0", classname), obj, ismethod, isvirtual)->accept(producer);
-    generateGetOutputs(subst("getNumOutputs$0", classname), obj, ismethod, isvirtual)->accept(producer);
+    generateGetInputs(in_fun + classname, obj, ismethod, funtype)->accept(producer);
+    generateGetOutputs(out_fun + classname, obj, ismethod, funtype)->accept(producer);
 }
 
 void CodeContainer::generateDAGLoopInternal(CodeLoop* loop, BlockInst* block, DeclareVarInst* count, bool omp)
@@ -367,10 +366,21 @@ void CodeContainer::processFIR(void)
     gGlobal->setVarType("inputs", Typed::kFloatMacro_ptr_ptr);
     gGlobal->setVarType("outputs", Typed::kFloatMacro_ptr_ptr);
     
-    // Types used in 'compute' prototype in -os mode
-    gGlobal->setVarType("iControl", Typed::kInt32_ptr);
-    gGlobal->setVarType("fControl", Typed::kFloatMacro_ptr);
+    // Type used in several methods using 'sample_rate' parameter
+    gGlobal->setVarType("sample_rate", Typed::kInt32);
     
+    /*
+        Used in SOUL backend and -os mode (C/C++)
+        18/08/22 : gGlobal->gOneSample == 3 fails because of typing
+        issues with iControl/fControl, so deactivated for now
+    */
+    if ((gGlobal->gOneSample >= 0 && gGlobal->gOneSample < 3) || gGlobal->gOneSampleControl) {
+        // Control is separated in the 'control()' function and iControl/fControl arrays
+        // are used to compute control related state to be used in 'run'
+        gGlobal->setVarType("iControl", Typed::kInt32_ptr);
+        gGlobal->setVarType("fControl", itfloatptr());
+    }
+      
     // Possibly add "fSamplingRate" field
     generateSR();
 
@@ -380,6 +390,117 @@ void CodeContainer::processFIR(void)
         set<CodeLoop*> visited;
         CodeLoop::groupSeqLoops(fCurLoop, visited);
     }
+ 
+    /*
+        Create memory layout, to be used in C++ backend and JSON generation.
+        The description order follows what will be done at allocation time.
+     
+        // Create static tables
+        mydsp::classInit();
+     
+        // Create DSP
+        dsp* DSP = mydsp::create();
+     
+        // Init DSP
+        DSP->instanceInit(44100);
+     */
+    if (gGlobal->gMemoryManager) {
+        {
+            // Compute DSP struct arrays size
+            StructInstVisitor struct_visitor;
+        
+            // Add the global static tables
+            fGlobalDeclarationInstructions->accept(&struct_visitor);
+        
+            // Compute R/W access for each subcontainer
+            ForLoopInst* loop = fCurLoop->generateScalarLoop("count");
+            loop->accept(&struct_visitor);
+            
+            // Subcontainers used in classInit
+            for (const auto& it : fSubContainers) {
+                // Check that the subcontainer name appears as a type name in fStaticInitInstructions
+                SearchSubcontainer search_class(it->getClassName());
+                fStaticInitInstructions->accept(&search_class);
+                if (search_class.fFound) {
+                    // Subcontainer size
+                    VariableSizeCounter struct_size(Address::kStruct);
+                    it->generateDeclarations(&struct_size);
+                    fMemoryLayout.push_back(make_tuple(it->getClassName(), int(Typed::kNoType), 0, struct_size.fSizeBytes, 0, 0));
+                    
+                    // Get the associated table size and access
+                    pair<string, int> field = gGlobal->gTablesSize[it->getClassName()];
+                    
+                    // Check the table name memory description
+                    MemoryDesc& decs = struct_visitor.getMemoryDesc(field.first);
+                    fMemoryLayout.push_back(make_tuple(field.first, int(Typed::kNoType), 0, field.second, decs.fRAccessCount, 0));
+                }
+            }
+        }
+        
+        {
+            // Compute DSP struct arrays size and R/W access
+            StructInstVisitor struct_visitor;
+        
+            // Add the DSP fields
+            fDeclarationInstructions->accept(&struct_visitor);
+            
+            // To generate R/W access in the DSP loop
+            ForLoopInst* loop = fCurLoop->generateScalarLoop("count");
+            loop->accept(&struct_visitor);
+            
+            // DSP object
+            int read_access = 0;
+            int write_access = 0;
+            for (const auto& it : struct_visitor.getFieldTable()) {
+                // Scalar types are kept in the DSP
+                if (it.second.fSize == 1) {
+                    read_access += it.second.fRAccessCount;
+                    write_access += it.second.fWAccessCount;
+                }
+            }
+            
+            // Array fields are transformed in pointers
+            ArrayToPointer array_pointer;
+            VariableSizeCounter struct_size(Address::kStruct);
+            array_pointer.getCode(fDeclarationInstructions)->accept(&struct_size);
+            fMemoryLayout.push_back(make_tuple(fKlassName,
+                                                int(Typed::kNoType),
+                                                0,
+                                                // Upper value : add virtual method pointer (8 bytes in 64 bits)
+                                                // + 8 bytes for memory alignment
+                                                struct_size.fSizeBytes + 8 + 8,
+                                                read_access,
+                                                write_access));
+            
+            // Arrays inside the DSP object
+            for (const auto& it : struct_visitor.getFieldTable()) {
+                // Arrays have size > 1
+                if (it.second.fSize > 1) {
+                    fMemoryLayout.push_back(make_tuple(it.first,
+                                                        int(it.second.fType),
+                                                        it.second.fSize,
+                                                        it.second.fSizeBytes,
+                                                        it.second.fRAccessCount,
+                                                        it.second.fWAccessCount));
+                }
+            }
+            
+            // Subcontainers used in instanceConstants
+            for (const auto& it : fSubContainers) {
+                // Check that the subcontainer name appears as a type name in fInitInstructions
+                SearchSubcontainer search_class(it->getClassName());
+                fInitInstructions->accept(&search_class);
+                if (search_class.fFound) {
+                    VariableSizeCounter struct_size(Address::kStruct);
+                    it->generateDeclarations(&struct_size);
+                    fMemoryLayout.push_back(make_tuple(it->getClassName(), int(Typed::kNoType), 0, struct_size.fSizeBytes, 0, 0));
+                }
+            }
+        }
+    }
+    
+    // Possibly generate JSON
+    generateJSONFile();
 
     // Sort struct fields by size and type
     // 05/16/17 : deactivated since it slows down the code...
@@ -474,6 +595,8 @@ void CodeContainer::printMacros(ostream& fout, int n)
             tab(n + 1, fout);
             fout << "#define FAUST_CLASS_NAME " << "\"" << fKlassName << "\"";
             tab(n + 1, fout);
+            fout << "#define FAUST_COMPILATION_OPIONS \"" << gGlobal->printCompilationOptions1() << "\"";
+            tab(n + 1, fout);
             fout << "#define FAUST_INPUTS " << fNumInputs;
             tab(n + 1, fout);
             fout << "#define FAUST_OUTPUTS " << fNumOutputs;
@@ -516,9 +639,9 @@ void CodeContainer::printMacros(ostream& fout, int n)
 // DSP API generation
 
 DeclareFunInst* CodeContainer::generateGetIO(const string& name, const string& obj, int io, bool ismethod,
-                                             bool isvirtual)
+                                             FunTyped::FunAttribute funtype)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -526,24 +649,23 @@ DeclareFunInst* CodeContainer::generateGetIO(const string& name, const string& o
     block->pushBackInst(InstBuilder::genRetInst(InstBuilder::genInt32NumInst(io)));
 
     // Creates function
-    FunTyped* fun_type = InstBuilder::genFunTyped(args, InstBuilder::genInt32Typed(),
-                                                  (isvirtual) ? FunTyped::kVirtual : FunTyped::kDefault);
+    FunTyped* fun_type = InstBuilder::genFunTyped(args, InstBuilder::genInt32Typed(), funtype);
     return InstBuilder::genDeclareFunInst(name, fun_type, block);
 }
 
-DeclareFunInst* CodeContainer::generateGetInputs(const string& name, const string& obj, bool ismethod, bool isvirtual)
+DeclareFunInst* CodeContainer::generateGetInputs(const string& name, const string& obj, bool ismethod, FunTyped::FunAttribute funtype)
 {
-    return generateGetIO(name, obj, fNumInputs, ismethod, isvirtual);
+    return generateGetIO(name, obj, fNumInputs, ismethod, funtype);
 }
 
-DeclareFunInst* CodeContainer::generateGetOutputs(const string& name, const string& obj, bool ismethod, bool isvirtual)
+DeclareFunInst* CodeContainer::generateGetOutputs(const string& name, const string& obj, bool ismethod, FunTyped::FunAttribute funtype)
 {
-    return generateGetIO(name, obj, fNumOutputs, ismethod, isvirtual);
+    return generateGetIO(name, obj, fNumOutputs, ismethod, funtype);
 }
 
 DeclareFunInst* CodeContainer::generateAllocate(const string& name, const string& obj, bool ismethod, bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -560,7 +682,7 @@ DeclareFunInst* CodeContainer::generateAllocate(const string& name, const string
 
 DeclareFunInst* CodeContainer::generateDestroy(const string& name, const string& obj, bool ismethod, bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -578,7 +700,7 @@ DeclareFunInst* CodeContainer::generateDestroy(const string& name, const string&
 DeclareFunInst* CodeContainer::generateGetIORate(const string& name, const string& obj, vector<int>& io, bool ismethod,
                                                  bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -617,7 +739,7 @@ DeclareFunInst* CodeContainer::generateGetIORate(const string& name, const strin
 DeclareFunInst* CodeContainer::generateInstanceClear(const string& name, const string& obj, bool ismethod,
                                                      bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -633,9 +755,9 @@ DeclareFunInst* CodeContainer::generateInstanceClear(const string& name, const s
 }
 
 DeclareFunInst* CodeContainer::generateInstanceConstants(const string& name, const string& obj, bool ismethod,
-                                                         bool isvirtual)
+                                                        bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -653,7 +775,7 @@ DeclareFunInst* CodeContainer::generateInstanceConstants(const string& name, con
 
 DeclareFunInst* CodeContainer::generateStaticInitFun(const string& name, bool isstatic)
 {
-    list<NamedTyped*> args;
+    Names args;
     args.push_back(InstBuilder::genNamedTyped("sample_rate", Typed::kInt32));
 
     BlockInst* block = InstBuilder::genBlockInst();
@@ -678,7 +800,7 @@ DeclareFunInst* CodeContainer::generateStaticInitFun(const string& name, bool is
 DeclareFunInst* CodeContainer::generateInstanceInitFun(const string& name, const string& obj, bool ismethod,
                                                        bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -699,7 +821,7 @@ DeclareFunInst* CodeContainer::generateInstanceInitFun(const string& name, const
 
 DeclareFunInst* CodeContainer::generateFillFun(const string& name, const string& obj, bool ismethod, bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -713,7 +835,7 @@ DeclareFunInst* CodeContainer::generateFillFun(const string& name, const string&
     BlockInst* block = InstBuilder::genBlockInst();
     block->pushBackInst(fComputeBlockInstructions);
     // Hack for Julia
-    if (gGlobal->gOutputLang == "julia") {
+    if (gGlobal->gOutputLang == "julia" || gGlobal->gOutputLang == "jax") {
         block->pushBackInst(fCurLoop->generateSimpleScalarLoop("count"));
     } else {
         block->pushBackInst(fCurLoop->generateScalarLoop("count"));
@@ -728,7 +850,7 @@ DeclareFunInst* CodeContainer::generateFillFun(const string& name, const string&
 
 DeclareFunInst* CodeContainer::generateInit(const string& name, const string& obj, bool ismethod, bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -736,7 +858,7 @@ DeclareFunInst* CodeContainer::generateInit(const string& name, const string& ob
 
     BlockInst* block = InstBuilder::genBlockInst();
     {
-        list<ValueInst*> args1;
+        Values args1;
         if (!ismethod) {
             args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
@@ -745,7 +867,7 @@ DeclareFunInst* CodeContainer::generateInit(const string& name, const string& ob
     }
 
     {
-        list<ValueInst*> args1;
+        Values args1;
         if (!ismethod) {
             args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
@@ -760,7 +882,7 @@ DeclareFunInst* CodeContainer::generateInit(const string& name, const string& ob
 DeclareFunInst* CodeContainer::generateInstanceInit(const string& name, const string& obj, bool ismethod,
                                                     bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -768,7 +890,7 @@ DeclareFunInst* CodeContainer::generateInstanceInit(const string& name, const st
 
     BlockInst* block = InstBuilder::genBlockInst();
     {
-        list<ValueInst*> args1;
+        Values args1;
         if (!ismethod) {
             args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
@@ -777,7 +899,7 @@ DeclareFunInst* CodeContainer::generateInstanceInit(const string& name, const st
     }
 
     {
-        list<ValueInst*> args1;
+        Values args1;
         if (!ismethod) {
             args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
@@ -785,7 +907,7 @@ DeclareFunInst* CodeContainer::generateInstanceInit(const string& name, const st
     }
 
     {
-        list<ValueInst*> args1;
+        Values args1;
         if (!ismethod) {
             args1.push_back(InstBuilder::genLoadFunArgsVar(obj));
         }
@@ -799,7 +921,7 @@ DeclareFunInst* CodeContainer::generateInstanceInit(const string& name, const st
 DeclareFunInst* CodeContainer::generateGetSampleRate(const string& name, const string& obj, bool ismethod,
                                                      bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -815,7 +937,7 @@ DeclareFunInst* CodeContainer::generateGetSampleRate(const string& name, const s
 
 DeclareFunInst* CodeContainer::generateComputeFun(const string& name, const string& obj, bool ismethod, bool isvirtual)
 {
-    list<NamedTyped*> args;
+    Names args;
     if (!ismethod) {
         args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
     }
@@ -837,7 +959,7 @@ DeclareFunInst* CodeContainer::generateComputeFun(const string& name, const stri
 
 DeclareFunInst* CodeContainer::generateCalloc()
 {
-    list<NamedTyped*> args;
+    Names args;
     args.push_back(InstBuilder::genNamedTyped("count", Typed::kInt64));
     args.push_back(InstBuilder::genNamedTyped("size", Typed::kInt64));
 
@@ -849,7 +971,7 @@ DeclareFunInst* CodeContainer::generateCalloc()
 
 DeclareFunInst* CodeContainer::generateFree()
 {
-    list<NamedTyped*> args;
+    Names args;
     args.push_back(InstBuilder::genNamedTyped("ptr", Typed::kVoid_ptr));
 
     // Creates function
@@ -859,11 +981,11 @@ DeclareFunInst* CodeContainer::generateFree()
 
 DeclareFunInst* CodeContainer::generateNewDsp(const string& name, int size)
 {
-    list<NamedTyped*> args;
+    Names args;
 
     BlockInst* block = InstBuilder::genBlockInst();
     {
-        list<ValueInst*> args1;
+        Values args1;
         args1.push_back(InstBuilder::genInt64NumInst(1));
         args1.push_back(InstBuilder::genInt64NumInst(size));
         block->pushBackInst(InstBuilder::genRetInst(InstBuilder::genCastInst(
@@ -877,12 +999,12 @@ DeclareFunInst* CodeContainer::generateNewDsp(const string& name, int size)
 
 DeclareFunInst* CodeContainer::generateDeleteDsp(const string& name, const string& obj)
 {
-    list<NamedTyped*> args;
+    Names args;
     args.push_back(InstBuilder::genNamedTyped(obj, Typed::kObj_ptr));
 
     BlockInst* block = InstBuilder::genBlockInst();
     {
-        list<ValueInst*> args1;
+        Values args1;
         args1.push_back(InstBuilder::genCastInst(InstBuilder::genLoadFunArgsVar(obj),
                                                 InstBuilder::genBasicTyped(Typed::kVoid_ptr)));
         block->pushBackInst(InstBuilder::genDropInst(InstBuilder::genFunCallInst("free", args1)));
@@ -895,3 +1017,19 @@ DeclareFunInst* CodeContainer::generateDeleteDsp(const string& name, const strin
     FunTyped* fun_type = InstBuilder::genFunTyped(args, InstBuilder::genBasicTyped(Typed::kVoid), FunTyped::kLocal);
     return InstBuilder::genDeclareFunInst(name, fun_type, block);
 }
+
+void CodeContainer::generateJSONFile()
+{
+    // Generate JSON (which checks for non duplicated path)
+    if (gGlobal->gPrintJSONSwitch) {
+        if (gGlobal->gFloatSize == 1) {
+            generateJSONFile<float>();
+        } else {
+            generateJSONFile<double>();
+        }
+    } else {
+        // Checks for non duplicated path
+        JSONInstVisitor<float> path_checker;
+        generateUserInterface(&path_checker);
+    }
+}
