@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -175,7 +175,7 @@ void Compiler::generateUserInterfaceTree(Tree t, bool root)
         // Empty labels will be renamed with a 0xABCD (address) kind of name that is ignored and not displayed by UI
         // architectures
         const char* str = tree2str(right(label));
-        const char* model;
+        const char* model = nullptr;
 
         // extract metadata from group label str resulting in a simplifiedLabel
         // and metadata declarations for fictive zone at address 0
@@ -212,7 +212,9 @@ void Compiler::generateUserInterfaceTree(Tree t, bool root)
                 fJSON.openTabBox(group.c_str());
                 break;
             default:
-                throw faustexception("ERROR in user interface generation 1\n");
+                cerr << "ERROR : user interface generation 1\n";
+                faustassert(false);
+                break;
         }
 
         fClass->addUICode(subst(model, group));
@@ -223,7 +225,8 @@ void Compiler::generateUserInterfaceTree(Tree t, bool root)
     } else if (isUiWidget(t, label, varname, sig)) {
         generateWidgetCode(label, varname, sig);
     } else {
-        throw faustexception("ERROR in user interface generation 2\n");
+        cerr << "ERROR : user interface generation 2\n";
+        faustassert(false);
     }
 }
 
@@ -335,7 +338,8 @@ void Compiler::generateWidgetCode(Tree fulllabel, Tree varname, Tree sig)
         fJSON.addSoundfile(checkNullLabel(varname, label).c_str(),
                            ((url == "") ? prepareURL(label).c_str() : url.c_str()), NULL);
     } else {
-        throw faustexception("ERROR in generating widget code 3\n");
+        cerr << "ERROR : generating widget code 3\n";
+        faustassert(false);
     }
 }
 
@@ -358,7 +362,8 @@ void Compiler::generateMacroInterfaceTree(const string& pathname, Tree t)
     } else if (isUiWidget(t, label, varname, sig)) {
         generateWidgetMacro(pathname, label, varname, sig);
     } else {
-        throw faustexception("ERROR in user interface macro generation 2\n");
+        cerr << "ERROR : user interface macro generation 2\n";
+        faustassert(false);
     }
 }
 
@@ -416,6 +421,7 @@ void Compiler::generateWidgetMacro(const string& pathname, Tree fulllabel, Tree
         fClass->addUIMacro(subst("FAUST_ADDSOUNDFILE(\"$0\", $1);", pathlabel, tree2str(varname)));
 
     } else {
-        throw faustexception("ERROR in generating widget macro\n");
+        cerr << "ERROR in generating widget macro\n";
+        faustassert(false);
     }
 }
