@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -30,6 +30,7 @@
 #include <iostream>
 #include <sstream>
 #include <vector>
+#include <climits>
 
 #include "compatibility.hh"
 #include "compile.hh"
@@ -39,14 +40,11 @@
 #include "prim2.hh"
 #include "privatise.hh"
 #include "recursivness.hh"
-#include "sigConstantPropagation.hh"
-#include "sigPromotion.hh"
 #include "sigToGraph.hh"
 #include "signal2vhdlVisitor.hh"
 #include "sigprint.hh"
 #include "sigtype.hh"
-#include "sigtyperules.hh"
-#include "simplify.hh"
+#include "normalform.hh"
 #include "timing.hh"
 #include "xtended.hh"
 
@@ -89,85 +87,57 @@ string ScalarCompiler::getFreshID(const string& prefix)
 
 Tree ScalarCompiler::prepare(Tree LS)
 {
-    startTiming("ScalarCompiler::prepare");
-    //  startTiming("first simplification");
-    //  LS = simplify(LS);
-    //  endTiming("first simplification");
-    startTiming("deBruijn2Sym");
-    Tree L1 = deBruijn2Sym(LS);  // convert debruijn recursion into symbolic recursion
-    endTiming("deBruijn2Sym");
-
-    startTiming("L1 typeAnnotation");
-    // Annotate L1 with type information (needed by castAndPromotion(), but don't check causality)
-    typeAnnotation(L1, gGlobal->gLocalCausalityCheck);
-    endTiming("L1 typeAnnotation");
-
-    startTiming("Cast and Promotion");
-    SignalPromotion SP;
-    // SP.trace(true, "Cast");
-    Tree L1b = SP.mapself(L1);
-    endTiming("Cast and Promotion");
-
-    startTiming("second simplification");
-    Tree L2 = simplify(L1b);  // Simplify by executing every computable operation
-    endTiming("second simplification");
-
-    startTiming("Constant propagation");
-    SignalConstantPropagation SK;
-    // SK.trace(true, "ConstProp2");
-    Tree L2b = SK.mapself(L2);
-    endTiming("Constant propagation");
-
-    startTiming("privatise");
-    Tree L3 = privatise(L2b);  // Un-share tables with multiple writers
-    endTiming("privatise");
-
-    startTiming("conditionAnnotation");
-    conditionAnnotation(L3);
-    endTiming("conditionAnnotation");
-    // conditionStatistics(L3); // Count condition occurrences
-
+    startTiming("prepare");
+    Tree L1 = simplifyToNormalForm(LS);
+    
     // dump normal form
-    if (gGlobal->gDumpNorm) {
-        cout << ppsig(L3) << endl;
+    if (gGlobal->gDumpNorm == 0) {
+        cout << ppsig(L1) << endl;
         throw faustexception("Dump normal form finished...\n");
+    } else if (gGlobal->gDumpNorm == 1) {
+        ppsigShared(L1, cout);
+        throw faustexception("Dump shared normal form finished...\n");
     }
-
+    
+    startTiming("privatise");
+    Tree L2 = privatise(L1);  // Un-share tables with multiple writers
+    endTiming("privatise");
+    
+    startTiming("conditionAnnotation");
+    conditionAnnotation(L2);
+    endTiming("conditionAnnotation");
+    
     startTiming("recursivnessAnnotation");
-    recursivnessAnnotation(L3);  // Annotate L3 with recursivness information
+    recursivnessAnnotation(L2);  // Annotate L2 with recursivness information
     endTiming("recursivnessAnnotation");
-
-    startTiming("typeAnnotation");
-    typeAnnotation(L3, true);    // Annotate L3 with type information
-    endTiming("typeAnnotation");
-
+    
+    startTiming("L2 typeAnnotation");
+    typeAnnotation(L2, true);     // Annotate L2 with type information and check causality
+    endTiming("L2 typeAnnotation");
+    
     startTiming("sharingAnalysis");
-    sharingAnalysis(L3);         // Annotate L3 with sharing count
+    sharingAnalysis(L2);         // Annotate L2 with sharing count
     endTiming("sharingAnalysis");
-
+    
     startTiming("occurrences analysis");
     delete fOccMarkup;
     fOccMarkup = new old_OccMarkup(fConditionProperty);
-    fOccMarkup->mark(L3);  // Annotate L3 with occurrences analysis
+    fOccMarkup->mark(L2);        // Annotate L2 with occurrences analysis
     endTiming("occurrences analysis");
-
-    endTiming("ScalarCompiler::prepare");
-
+    
+    endTiming("prepare");
+    
     if (gGlobal->gDrawSignals) {
         ofstream dotfile(subst("$0-sig.dot", gGlobal->makeDrawPath()).c_str());
-        sigToGraph(L3, dotfile);
+        sigToGraph(L2, dotfile);
     }
-
-    // Generate VHDL if --vhdl option is set
+    
+    // Generate VHDL if -vhdl option is set
     if (gGlobal->gVHDLSwitch) {
-        Signal2VHDLVisitor V(fOccMarkup);
-        ofstream dotfile(subst("faust.vhd", gGlobal->makeDrawPath()).c_str());
-        V.sigToVHDL(L3, dotfile);
-        V.trace(gGlobal->gVHDLTrace, "VHDL");  // activate with --trace option
-        V.mapself(L3);
+        sigVHDLFile(fOccMarkup, L2, gGlobal->gVHDLTrace);
     }
-
-    return L3;
+    
+    return L2;
 }
 
 Tree ScalarCompiler::prepare2(Tree L0)
@@ -365,7 +335,7 @@ void ScalarCompiler::compileMultiSignal(Tree L)
     for (int i = 0; isList(L); L = tl(L), i++) {
         Tree sig = hd(L);
         fClass->addExecCode(
-            Statement("", subst("output$0[i] = $2$1;", T(i), generateCacheCode(sig, CS(sig)), xcast())));
+            Statement("", subst("output$0[i] = $2($1);", T(i), generateCacheCode(sig, CS(sig)), xcast())));
     }
 
     generateMetaData();
@@ -442,11 +412,7 @@ string ScalarCompiler::generateCode(Tree sig)
         return generateDelay(sig, x, y);
     } else if (isSigPrefix(sig, x, y)) {
         return generatePrefix(sig, x, y);
-    } else if (isSigIota(sig, x)) {
-        return generateIota(sig, x);
-    }
-
-    else if (isSigBinOp(sig, &i, x, y)) {
+    } else if (isSigBinOp(sig, &i, x, y)) {
         return generateBinOp(sig, i, x, y);
     } else if (isSigFFun(sig, ff, largs)) {
         return generateFFun(sig, ff, largs);
@@ -467,7 +433,7 @@ string ScalarCompiler::generateCode(Tree sig)
     else if (isSigSelect2(sig, sel, x, y)) {
         return generateSelect2(sig, sel, x, y);
     }
-    
+
     else if (isSigGen(sig, x)) {
         return generateSigGen(sig, x);
     }
@@ -475,7 +441,7 @@ string ScalarCompiler::generateCode(Tree sig)
     else if (isProj(sig, &i, x)) {
         return generateRecProj(sig, x, i);
     }
-    
+
     else if (isSigIntCast(sig, x)) {
         return generateIntCast(sig, x);
     } else if (isSigFloatCast(sig, x)) {
@@ -519,19 +485,18 @@ string ScalarCompiler::generateCode(Tree sig)
             throw faustexception("ERROR : 'control/enable' can only be used in scalar mode\n");
         }
         return generateControl(sig, x, y);
-	
+
     } else if (isSigAssertBounds(sig, x, y, z)) {
         /* no debug option for the moment */
         return generateCode(z);
     } else if (isSigLowest(sig, x) || isSigHighest(sig, x)) {
         throw faustexception("ERROR : annotations should have been deleted in Simplification process\n");
     }
-    
+
     /* we should not have any control at this stage*/
     else {
-        stringstream error;
-        error << "ERROR when compiling, ScalarCompiler::generateCode unrecognized signal : " << ppsig(sig) << endl;
-        throw faustexception(error.str());
+        cerr << "ERROR : when compiling, unrecognized signal : " << ppsig(sig) << endl;
+        faustassert(false);
     }
     return "error in generated code";
 }
@@ -607,41 +572,52 @@ string ScalarCompiler::generateOutput(Tree sig, const string& idx, const string&
     return dst;
 }
 
+static int binopPriority(Tree sig)
+{
+    int  opcode;
+    Tree arg1;
+    Tree arg2;
+    return isSigBinOp(sig, &opcode, arg1, arg2) ? gBinOpTable[opcode]->fPriority : INT_MAX;
+}
 /*****************************************************************************
  BINARY OPERATION
  *****************************************************************************/
 
 string ScalarCompiler::generateBinOp(Tree sig, int opcode, Tree arg1, Tree arg2)
 {
+    // check the priorities and add parentheses when needed
+    int p0 = gBinOpTable[opcode]->fPriority;
+    int p1 = binopPriority(arg1);
+    int p2 = binopPriority(arg2);
+    
+    bool np1 = (p0 > p1) || isLogicalOpcode(opcode);
+    bool np2 = (p0 > p2) || isLogicalOpcode(opcode);
+
+    string c1 = CS(arg1);
+    string c2 = CS(arg2);
+
     if (opcode == kDiv) {
         // special handling for division, we always want a float division
         Type t1 = getCertifiedSigType(arg1);
         Type t2 = getCertifiedSigType(arg2);
 
-        interval j = t2->getInterval();
-
-        if (j.haszero()) {
-            // potential division by zero
-            // interval    i = t1->getInterval();
-            // std::cerr << "WARNING : potential division by zero (" << i << "/" << j << ") in " << ppsig(sig) <<
-            // std::endl;
-        }
-
         if (t1->nature() == kInt && t2->nature() == kInt) {
-            return generateCacheCode(
-                sig, subst("($3($0) $1 $3($2))", CS(arg1), gBinOpTable[opcode]->fName, CS(arg2), ifloat()));
+            return generateCacheCode(sig, subst("($3($0) $1 $3($2))", c1, gBinOpTable[opcode]->fName, c2, ifloat()));
         } else if (t1->nature() == kInt && t2->nature() == kReal) {
-            return generateCacheCode(sig,
-                                     subst("($3($0) $1 $2)", CS(arg1), gBinOpTable[opcode]->fName, CS(arg2), ifloat()));
+            if (np2) c2 = subst("($0)", c2);
+            return generateCacheCode(sig, subst("($3($0) $1 $2)", c1, gBinOpTable[opcode]->fName, c2, ifloat()));
         } else if (t1->nature() == kReal && t2->nature() == kInt) {
-            return generateCacheCode(sig,
-                                     subst("($0 $1 $3($2))", CS(arg1), gBinOpTable[opcode]->fName, CS(arg2), ifloat()));
+            if (np1) c1 = subst("($0)", c1);
+            return generateCacheCode(sig, subst("($0 $1 $3($2))", c1, gBinOpTable[opcode]->fName, c2, ifloat()));
         } else {
-            return generateCacheCode(sig,
-                                     subst("($0 $1 $2)", CS(arg1), gBinOpTable[opcode]->fName, CS(arg2), ifloat()));
+            if (np1) c1 = subst("($0)", c1);
+            if (np2) c2 = subst("($0)", c2);
+            return generateCacheCode(sig, subst("($0 $1 $2)", c1, gBinOpTable[opcode]->fName, c2, ifloat()));
         }
     } else {
-        return generateCacheCode(sig, subst("($0 $1 $2)", CS(arg1), gBinOpTable[opcode]->fName, CS(arg2)));
+        if (np1) c1 = subst("($0)", c1);
+        if (np2) c2 = subst("($0)", c2);
+        return generateCacheCode(sig, subst("($0 $1 $2)", c1, gBinOpTable[opcode]->fName, c2));
     }
 }
 
@@ -711,12 +687,10 @@ string ScalarCompiler::generateCacheCode(Tree sig, const string& exp)
         return exp;
 
     } else {
-        stringstream error;
-        error << "ERROR in sharing count (" << sharing << ") for " << *sig << endl;
-        throw faustexception(error.str());
+        cerr << "ERROR : sharing count (" << sharing << ") for " << *sig << endl;
+        faustassert(false);
+        return {};
     }
-
-    return "Error in generateCacheCode";
 }
 
 // like generateCacheCode but we force caching like if sharing was always > 1
@@ -979,7 +953,7 @@ string ScalarCompiler::generateTable(Tree sig, Tree tsize, Tree content)
     int size;
     if (!isSigInt(tsize, &size)) {
         stringstream error;
-        error << "ERROR in generateTable : " << *tsize << " is not an integer expression " << endl;
+        error << "ERROR : generateTable : " << *tsize << " is not an integer expression " << endl;
         throw faustexception(error.str());
     }
 
@@ -1001,7 +975,7 @@ string ScalarCompiler::generateTable(Tree sig, Tree tsize, Tree content)
     // definition du nom et du type de la table
     // A REVOIR !!!!!!!!!
     Type t = getCertifiedSigType(content);  //, tEnv);
-    
+
     if (t->nature() == kInt) {
         vname = getFreshID("itbl");
         ctype = "int";
@@ -1027,7 +1001,7 @@ string ScalarCompiler::generateStaticTable(Tree sig, Tree tsize, Tree content)
     int size;
     if (!isSigInt(tsize, &size)) {
         stringstream error;
-        error << "ERROR in generateStaticTable : " << *tsize << " is not an integer expression " << endl;
+        error << "ERROR : generateStaticTable : " << *tsize << " is not an integer expression " << endl;
         throw faustexception(error.str());
     }
 
@@ -1053,7 +1027,7 @@ string ScalarCompiler::generateStaticTable(Tree sig, Tree tsize, Tree content)
     // definition du nom et du type de la table
     // A REVOIR !!!!!!!!!
     Type t = getCertifiedSigType(content);  //, tEnv);
-    
+
     if (t->nature() == kInt) {
         vname = getFreshID("itbl");
         ctype = "int";
@@ -1203,18 +1177,25 @@ string ScalarCompiler::generateControl(Tree sig, Tree x, Tree y)
 
 string ScalarCompiler::generatePrefix(Tree sig, Tree x, Tree e)
 {
-    Type te = getCertifiedSigType(sig);  //, tEnv);
-
-    string vperm = getFreshID("M");
-    string vtemp = getFreshID("T");
-
-    string type = (te->nature() == kInt) ? "int" : ifloat();
+    string vperm = getFreshID("pfPerm");
+    string vtemp = getFreshID("pfTemp");
+    string type = (getCertifiedSigType(sig)->nature() == kInt) ? "int" : ifloat();
 
     fClass->addDeclCode(subst("$0 \t$1;", type, vperm));
     fClass->addInitCode(subst("$0 = $1;", vperm, CS(x)));
-    fClass->addInitCode(subst("$0 \t$1;", type, vtemp));
 
-    fClass->addExecCode(Statement(getConditionCode(sig), subst("$0 = $1;", vtemp, vperm)));
+    fClass->addExecCode(Statement(getConditionCode(sig), subst("$0 \t$1 = $2;", type, vtemp, vperm)));
+    
+    /*
+    string res = CS(e);
+    string vname;
+    if (getVectorNameProperty(e, vname)) {
+        setVectorNameProperty(sig, vname);
+    } else {
+        faustassert(false);
+    }
+    */
+    
     fClass->addExecCode(Statement(getConditionCode(sig), subst("$0 = $1;", vperm, CS(e))));
     return vtemp;
 }
@@ -1228,26 +1209,6 @@ static bool isPowerOf2(int n)
     return !(n & (n - 1));
 }
 
-string ScalarCompiler::generateIota(Tree sig, Tree n)
-{
-    int size;
-    if (!isSigInt(n, &size)) {
-        throw faustexception("ERROR in generateIota\n");
-    }
-
-    string vperm = getFreshID("iota");
-
-    fClass->addDeclCode(subst("int \t$0;", vperm));
-    fClass->addClearCode(subst("$0 = 0;", vperm));
-
-    if (isPowerOf2(size)) {
-        fClass->addExecCode(Statement("", subst("$0 = ($0+1)&$1;", vperm, T(size - 1))));
-    } else {
-        fClass->addExecCode(Statement("", subst("if (++$0 == $1) $0=0;", vperm, T(size))));
-    }
-    return vperm;
-}
-
 /*****************************************************************************
  SELECT
  *****************************************************************************/
@@ -1273,9 +1234,9 @@ string ScalarCompiler::generateXtended(Tree sig)
     }
 
     if (p->needCache()) {
-        return generateCacheCode(sig, p->old_generateCode(fClass, args, types));
+        return generateCacheCode(sig, p->generateCode(fClass, args, types));
     } else {
-        return p->old_generateCode(fClass, args, types);
+        return p->generateCode(fClass, args, types);
     }
 }
 
@@ -1330,8 +1291,8 @@ string ScalarCompiler::generateDelay(Tree sig, Tree exp, Tree delay)
             // cerr << "it is a pure zero delay : " << code << endl;
             return code;
         } else {
-            cerr << "No vector name for : " << ppsig(exp) << endl;
-            faustassert(0);
+            cerr << "ERROR : no vector name for : " << ppsig(exp) << endl;
+            faustassert(false);
         }
     }
 
