@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -39,7 +39,7 @@ dsp_factory_base* DLangCodeContainer::produceFactory()
 
 CodeContainer* DLangCodeContainer::createScalarContainer(const string& name, int sub_container_type)
 {
-    return (gGlobal->gOneSample)
+    return (gGlobal->gOneSample >= 0)
         ? new DLangScalarOneSampleCodeContainer(name, "", 0, 1, fOut, sub_container_type)
         : new DLangScalarCodeContainer(name, "", 0, 1, fOut, sub_container_type);
 }
@@ -49,9 +49,6 @@ CodeContainer* DLangCodeContainer::createContainer(const string& name, const str
 {
     CodeContainer* container;
 
-    if (gGlobal->gMemoryManager) {
-        throw faustexception("ERROR : -mem not supported for D\n");
-    }
     if (gGlobal->gFloatSize == 3) {
         throw faustexception("ERROR : quad format not supported for D\n");
     }
@@ -69,7 +66,7 @@ CodeContainer* DLangCodeContainer::createContainer(const string& name, const str
     } else if (gGlobal->gVectorSwitch) {
         container = new DLangVectorCodeContainer(name, super, numInputs, numOutputs, dst);
     } else {
-        container = (gGlobal->gOneSample)
+        container = (gGlobal->gOneSample >= 0)
             ? new DLangScalarOneSampleCodeContainer(name, super, numInputs, numOutputs, dst, kInt)
             : new DLangScalarCodeContainer(name, super, numInputs, numOutputs, dst, kInt);
     }
@@ -83,7 +80,7 @@ void DLangCodeContainer::produceMetadata(int tabs)
     *fOut << "void metadata(Meta* m) nothrow @nogc { ";
 
     // We do not want to accumulate metadata from all hierachical levels, so the upper level only is kept
-    for (auto& i : gGlobal->gMetaDataSet) {
+    for (const auto& i : gGlobal->gMetaDataSet) {
         if (i.first != tree("author")) {
             tab(tabs + 1, *fOut);
             *fOut << "m.declare(\"" << *(i.first) << "\", " << **(i.second.begin()) << ");";
@@ -173,16 +170,11 @@ void DLangCodeContainer::produceInternal()
     *fOut << "struct " << fKlassName << " {\n";
     *fOut << "nothrow:\n";
     *fOut << "@nogc:";
-
     tab(n + 1, *fOut);
 
-    if (gGlobal->gUIMacroSwitch) {
-        tab(n, *fOut);
-        *fOut << "  public:";
-    } else {
-        tab(n, *fOut);
-        *fOut << "  private:";
-    }
+    tab(n, *fOut);
+    *fOut << "  private:";
+    
     tab(n + 1, *fOut);
     tab(n + 1, *fOut);
 
@@ -197,7 +189,7 @@ void DLangCodeContainer::produceInternal()
     tab(n + 1, *fOut);
 
     // fKlassName used in method naming for subclasses
-    produceInfoFunctions(n + 1, fKlassName, "dsp", true, false, &fCodeProducer);
+    produceInfoFunctions(n + 1, fKlassName, "dsp", true, FunTyped::kDefault, &fCodeProducer);
 
     // TODO
     // generateInstanceInitFun("instanceInit" + fKlassName, true, false)->accept(&fCodeProducer);
@@ -249,8 +241,9 @@ string DLangCodeContainer::dModuleName(const string& klassName)
 {
     string moduleName = klassName;
     transform(moduleName.begin(), moduleName.end(), moduleName.begin(), ::tolower);
-    if (gGlobal->gNameSpace != "")
-        moduleName = gGlobal->gNameSpace + "." + moduleName;
+    if (gGlobal->gNamespace != "") {
+        moduleName = gGlobal->gNamespace + "." + moduleName;
+    }
     return moduleName;
 }
 
@@ -263,31 +256,23 @@ void DLangCodeContainer::produceClass()
 
     generateImports();
 
-    // Sub containers
+    // Generate gub containers
     generateSubContainers();
 
     *fOut << "alias FAUSTCLASS = " << fKlassName << ";" << endl;
   
     // Global declarations
-    tab(n, *fOut);
     fCodeProducer.Tab(n);
     generateGlobalDeclarations(&fCodeProducer);
 
     tab(n, *fOut);
     *fOut << "class " << fKlassName << " : " << fSuperKlassName << "\n{\n";
     *fOut << "nothrow:\n";
-    *fOut << "@nogc:\n";
-
-
+    *fOut << "@nogc:";
     tab(n + 1, *fOut);
 
-    if (gGlobal->gUIMacroSwitch) {
-        tab(n, *fOut);
-        *fOut << " public:";
-    } else {
-        tab(n, *fOut);
-        *fOut << " private:";
-    }
+    tab(n, *fOut);
+    *fOut << " private:";
     tab(n + 1, *fOut);
 
     // Fields
@@ -344,7 +329,7 @@ void DLangCodeContainer::produceClass()
 
     tab(n + 1, *fOut);
     // No class name for main class
-    produceInfoFunctions(n + 1, "", "dsp", true, true, &fCodeProducer);  // Inits
+    produceInfoFunctions(n + 1, "", "dsp", true, FunTyped::kVirtual, &fCodeProducer);  // Inits
 
     tab(n + 1, *fOut);
     *fOut << "static void classInit(int sample_rate) nothrow @nogc {";
@@ -412,9 +397,6 @@ void DLangCodeContainer::produceClass()
     tab(n, *fOut);
     tab(n, *fOut);
     *fOut << "}" << endl;
-
-    // Generate user interface macros if needed
-    printMacros(*fOut, n);
 }
 
 // Scalar
@@ -492,10 +474,8 @@ void DLangScalarOneSampleCodeContainer::produceClass()
     generateSubContainers();
 
     *fOut << "alias FAUSTCLASS = " << fKlassName << ";" << endl;
-    tab(n, *fOut);
-
+  
     // Global declarations
-    tab(n, *fOut);
     fCodeProducer.Tab(n);
     generateGlobalDeclarations(&fCodeProducer);
 
@@ -503,22 +483,16 @@ void DLangScalarOneSampleCodeContainer::produceClass()
     *fOut << "enum FAUST_OUTPUTS = " << fNumOutputs << ";" << endl;
     *fOut << "enum FAUST_INT_CONTROLS = " << fInt32ControlNum << ";" << endl;
     *fOut << "enum FAUST_REAL_CONTROLS = " << fRealControlNum << ";" << endl;
-    tab(n, *fOut);
     fSuperKlassName = "one_sample_dsp";
 
     tab(n, *fOut);
     *fOut << "class " << fKlassName << " : " << fSuperKlassName << "\n{\n";
     *fOut << "nothrow:\n";
-    *fOut << "@nogc:\n";
+    *fOut << "@nogc:";
     tab(n + 1, *fOut);
 
-    if (gGlobal->gUIMacroSwitch) {
-        tab(n, *fOut);
-        *fOut << " public:";
-    } else {
-        tab(n, *fOut);
-        *fOut << " private:";
-    }
+    tab(n, *fOut);
+    *fOut << " private:";
     tab(n + 1, *fOut);
 
     // Fields
@@ -575,7 +549,7 @@ void DLangScalarOneSampleCodeContainer::produceClass()
 
     tab(n + 1, *fOut);
     // No class name for main class
-    produceInfoFunctions(n + 1, "", "dsp", true, true, &fCodeProducer);  // Inits
+    produceInfoFunctions(n + 1, "", "dsp", true, FunTyped::kVirtual, &fCodeProducer);  // Inits
 
     tab(n + 1, *fOut);
     *fOut << "static void classInit(int sample_rate) nothrow @nogc {";
@@ -656,10 +630,7 @@ void DLangScalarOneSampleCodeContainer::produceClass()
     generateCompute(n);
     tab(n, *fOut);
     tab(n, *fOut);
-    *fOut << "};" << endl;
-
-    // Generate user interface macros if needed
-    printMacros(*fOut, n);
+    *fOut << "}" << endl;
 }
 
 // Vector
