@@ -4,22 +4,26 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
  ************************************************************************/
 
+#if defined(_WIN32)
+#include "compatibility.hh"
+#else // !defined(_WIN32)
 #include <libgen.h>
+#endif // defined(_WIN32)
 
 #include "Text.hh"
 #include "interpreter_dynamic_dsp_aux.hh"
@@ -73,7 +77,7 @@ LIBFAUST_API interpreter_dsp_factory* createInterpreterDSPFactoryFromString(cons
                 }
                 argv1[argc1] = nullptr;  // NULL terminated argv
                 
-                dsp_factory_base* dsp_factory_aux = createFactory(name_app.c_str(), dsp_content.c_str(), argc1, argv1, error_msg, true);
+                dsp_factory_base* dsp_factory_aux = createFactory(name_app, dsp_content, argc1, argv1, error_msg, true);
                 if (dsp_factory_aux) {
                     dsp_factory_aux->setName(name_app);
                     interpreter_dsp_factory* factory = new interpreter_dsp_factory(dsp_factory_aux);
@@ -110,7 +114,7 @@ LIBFAUST_API interpreter_dsp_factory* createInterpreterDSPFactoryFromSignals(con
         }
         argv1[argc1] = nullptr;  // NULL terminated argv
         
-        dsp_factory_base* dsp_factory_aux = createFactory(name_app.c_str(), signals, argc1, argv1, error_msg);
+        dsp_factory_base* dsp_factory_aux = createFactory(name_app, signals, argc1, argv1, error_msg);
         if (dsp_factory_aux) {
             dsp_factory_aux->setName(name_app);
             interpreter_dsp_factory* factory = new interpreter_dsp_factory(dsp_factory_aux);
