@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -52,7 +52,6 @@
 #include <llvm/Support/ManagedStatic.h>
 #include <llvm/Support/MemoryBuffer.h>
 #include <llvm/Support/SourceMgr.h>
-#include <llvm/Support/TargetRegistry.h>
 #include <llvm/Support/TargetSelect.h>
 #include <llvm/Support/Host.h>
 #include <llvm/Transforms/IPO.h>
@@ -65,7 +64,14 @@
 #include <llvm/Bitcode/BitcodeReader.h>
 #include <llvm/Bitcode/BitcodeWriter.h>
 
-#if defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || defined(LLVM_130) || defined(LLVM_140)
+// Disappears in LLVM 15
+#if LLVM_VERSION_MAJOR >= 14
+#include <llvm/MC/TargetRegistry.h>
+#else
+#include <llvm/Support/TargetRegistry.h>
+#endif
+
+#if LLVM_VERSION_MAJOR >= 10
 #include <llvm/InitializePasses.h>
 #include <llvm/Support/CodeGen.h>
 #endif
@@ -114,8 +120,7 @@ void llvm_dynamic_dsp_factory_aux::write(ostream* out, bool binary, bool small)
     string res;
     raw_string_ostream out_str(res);
     if (binary) {
-#if defined(LLVM_80) || defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || \
-    defined(LLVM_130) || defined(LLVM_140)
+#if LLVM_VERSION_MAJOR >= 8
         WriteBitcodeToFile(*fModule, out_str);
 #else
         WriteBitcodeToFile(fModule, out_str);
@@ -131,8 +136,7 @@ string llvm_dynamic_dsp_factory_aux::writeDSPFactoryToBitcode()
 {
     string res;
     raw_string_ostream out(res);
-#if defined(LLVM_80) || defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || \
-    defined(LLVM_130) || defined(LLVM_140)
+#if LLVM_VERSION_MAJOR >= 8
     WriteBitcodeToFile(*fModule, out);
 #else
     WriteBitcodeToFile(fModule, out);
@@ -149,8 +153,7 @@ bool llvm_dynamic_dsp_factory_aux::writeDSPFactoryToBitcodeFile(const string& bi
         cerr << "ERROR : writeDSPFactoryToBitcodeFile could not open file : " << err.message();
         return false;
     }
-#if defined(LLVM_80) || defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || \
-    defined(LLVM_130) || defined(LLVM_140)
+#if LLVM_VERSION_MAJOR >= 8
     WriteBitcodeToFile(*fModule, out);
 #else
     WriteBitcodeToFile(fModule, out);
@@ -266,13 +269,13 @@ bool llvm_dynamic_dsp_factory_aux::initJIT(string& error_msg)
 
     builder.setOptLevel(CodeGenOpt::Aggressive);
     builder.setEngineKind(EngineKind::JIT);
-#if defined(LLVM_50)
+#if LLVM_VERSION_MAJOR == 5
     builder.setCodeModel(CodeModel::JITDefault);
 #endif
 
     string buider_error;
     builder.setErrorStr(&buider_error);
-
+  
     string triple, cpu;
     splitTarget(fTarget, triple, cpu);
     fModule->setTargetTriple(triple);
@@ -288,12 +291,11 @@ bool llvm_dynamic_dsp_factory_aux::initJIT(string& error_msg)
     targetOptions.GuaranteedTailCallOpt = true;
     targetOptions.NoTrappingFPMath      = true;
     
-#if defined(LLVM_90) || defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || defined(LLVM_130) || \
-    defined(LLVM_140)
+#if LLVM_VERSION_MAJOR >= 9
     targetOptions.NoSignedZerosFPMath   = true;
 #endif
     
-#if defined(LLVM_110) || defined(LLVM_120) || defined(LLVM_130) || defined(LLVM_140)
+#if LLVM_VERSION_MAJOR >= 11
     targetOptions.setFPDenormalMode(DenormalMode::getIEEE());
 #else
     targetOptions.FPDenormalMode = FPDenormal::IEEE;
@@ -303,7 +305,7 @@ bool llvm_dynamic_dsp_factory_aux::initJIT(string& error_msg)
     
     string debug_var = (getenv("FAUST_DEBUG")) ? string(getenv("FAUST_DEBUG")) : "";
     if ((debug_var != "") && (debug_var.find("FAUST_LLVM3") != string::npos)) {
-#if !defined(LLVM_120) && !defined(LLVM_130) && !defined(LLVM_140)
+#if LLVM_VERSION_MAJOR < 12
         targetOptions.PrintMachineCode = true;
 #endif
     }
@@ -338,7 +340,7 @@ bool llvm_dynamic_dsp_factory_aux::initJIT(string& error_msg)
         }
 
         if ((debug_var != "") && (debug_var.find("FAUST_LLVM1") != string::npos)) {
-    #if defined(LLVM_50)
+    #if LLVM_VERSION_MAJOR == 5
             TargetRegistry::printRegisteredTargetsForVersion();
     #endif
             dumpLLVM(fModule);
@@ -382,10 +384,13 @@ static llvm_dsp_factory* readDSPFactoryFromBitcodeAux(MEMORY_BUFFER buffer, cons
     } else {
         try {
             LLVMContext* context = new LLVMContext();
+#if LLVM_VERSION_MAJOR == 15
+            context->setOpaquePointers(false);
+#endif
             Module*      module  = ParseBitcodeFile(buffer, *context, error_msg);
             if (!module) return nullptr;
-            llvm_dynamic_dsp_factory_aux* factory_aux =
-                new llvm_dynamic_dsp_factory_aux(sha_key, module, context, target, opt_level);
+            llvm_dynamic_dsp_factory_aux* factory_aux
+                = new llvm_dynamic_dsp_factory_aux(sha_key, module, context, target, opt_level);
             if (factory_aux->initJIT(error_msg)) {
                 llvm_dsp_factory* factory = new llvm_dsp_factory(factory_aux);
                 llvm_dsp_factory_aux::gLLVMFactoryTable.setFactory(factory);
@@ -433,9 +438,9 @@ bool llvm_dynamic_dsp_factory_aux::writeDSPFactoryToObjectcodeFileAux(const stri
     }
 
     legacy::PassManager pass;
-#if defined(LLVM_100) || defined(LLVM_110) || defined(LLVM_120) || defined(LLVM_130) || defined(LLVM_140)
+#if LLVM_VERSION_MAJOR >= 10
     if (TheTargetMachine->addPassesToEmitFile(pass, dest, nullptr, CGFT_ObjectFile)) {
-#elif defined(LLVM_80) || defined(LLVM_90)
+#elif LLVM_VERSION_MAJOR >= 8
     if (TheTargetMachine->addPassesToEmitFile(pass, dest, nullptr, TargetMachine::CGFT_ObjectFile)) {
 #else
     if (TheTargetMachine->addPassesToEmitFile(pass, dest, TargetMachine::CGFT_ObjectFile, true)) {
@@ -489,6 +494,9 @@ static llvm_dsp_factory* readDSPFactoryFromIRAux(MEMORY_BUFFER buffer, const str
             }
             setlocale(LC_ALL, "C");
             LLVMContext* context = new LLVMContext();
+#if LLVM_VERSION_MAJOR == 15
+            context->setOpaquePointers(false);
+#endif
             SMDiagnostic err;
             // parseIR takes ownership of the given buffer, so don't delete it
             Module* module = parseIR(buffer, err, *context).release();
@@ -573,7 +581,7 @@ Module* linkAllModules(LLVMContext* context, Module* dst, string& error)
         
 // Public C++ API
 
-EXPORT llvm_dsp_factory* createDSPFactoryFromFile(const string& filename, int argc, const char* argv[],
+LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromFile(const string& filename, int argc, const char* argv[],
                                                   const string& target, string& error_msg, int opt_level)
 {
     string base = basename((char*)filename.c_str());
@@ -588,9 +596,9 @@ EXPORT llvm_dsp_factory* createDSPFactoryFromFile(const string& filename, int ar
     }
 }
 
-EXPORT llvm_dsp_factory* createDSPFactoryFromString(const string& name_app, const string& dsp_content, int argc,
-                                                    const char* argv[], const string& target, string& error_msg,
-                                                    int opt_level)
+LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromString(const string& name_app, const string& dsp_content, int argc,
+                                                       const char* argv[], const string& target, string& error_msg,
+                                                       int opt_level)
 {
     LOCK_API
     string expanded_dsp_content, sha_key;
@@ -620,25 +628,26 @@ EXPORT llvm_dsp_factory* createDSPFactoryFromString(const string& name_app, cons
                 argv1[argc1] = nullptr;  // NULL terminated argv
                 
                 llvm_dynamic_dsp_factory_aux* factory_aux
-                    = static_cast<llvm_dynamic_dsp_factory_aux*>(createFactory(name_app.c_str(),
-                                                                               dsp_content.c_str(),
-                                                                               argc1, argv1,
-                                                                               error_msg,
-                                                                               true));
-                if (factory_aux && factory_aux->initJIT(error_msg)) {
+                    = static_cast<llvm_dynamic_dsp_factory_aux*>(createFactory(name_app,
+                                                                            dsp_content,
+                                                                            argc1, argv1,
+                                                                            error_msg,
+                                                                            true));
+                if (factory_aux) {
                     factory_aux->setTarget(target);
                     factory_aux->setOptlevel(opt_level);
                     factory_aux->setClassName(getParam(argc, argv, "-cn", "mydsp"));
                     factory_aux->setName(name_app);
-                    llvm_dsp_factory* factory = new llvm_dsp_factory(factory_aux);
-                    llvm_dsp_factory_aux::gLLVMFactoryTable.setFactory(factory);
-                    factory->setSHAKey(sha_key);
-                    factory->setDSPCode(expanded_dsp_content);
-                    return factory;
-                } else {
-                    delete factory_aux;
-                    return nullptr;
+                    if (factory_aux->initJIT(error_msg)) {
+                        llvm_dsp_factory* factory = new llvm_dsp_factory(factory_aux);
+                        llvm_dsp_factory_aux::gLLVMFactoryTable.setFactory(factory);
+                        factory->setSHAKey(sha_key);
+                        factory->setDSPCode(expanded_dsp_content);
+                        return factory;
+                    }
                 }
+                delete factory_aux;
+                return nullptr;
             } catch (faustexception& e) {
                 error_msg = e.what();
                 return nullptr;
@@ -647,11 +656,11 @@ EXPORT llvm_dsp_factory* createDSPFactoryFromString(const string& name_app, cons
     }
 }
         
-EXPORT llvm_dsp_factory* createDSPFactoryFromSignals(const std::string& name_app, tvec signals,
-                                                    int argc, const char* argv[],
-                                                    const std::string& target,
-                                                    std::string& error_msg,
-                                                    int opt_level)
+LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromSignals(const string& name_app, tvec signals,
+                                                        int argc, const char* argv[],
+                                                        const string& target,
+                                                        string& error_msg,
+                                                        int opt_level)
 {
     LOCK_API
     try {
@@ -662,14 +671,14 @@ EXPORT llvm_dsp_factory* createDSPFactoryFromSignals(const std::string& name_app
         argv1[argc1++] = "llvm";
         argv1[argc1++] = "-o";
         argv1[argc1++] = "string";
-            // Copy arguments
+        // Copy arguments
         for (int i = 0; i < argc; i++) {
             argv1[argc1++] = argv[i];
         }
         argv1[argc1] = nullptr;  // NULL terminated argv
         
         llvm_dynamic_dsp_factory_aux* factory_aux
-            = static_cast<llvm_dynamic_dsp_factory_aux*>(createFactory(name_app.c_str(), signals, argc1, argv1, error_msg));
+            = static_cast<llvm_dynamic_dsp_factory_aux*>(createFactory(name_app, signals, argc1, argv1, error_msg));
         if (factory_aux && factory_aux->initJIT(error_msg)) {
             factory_aux->setTarget(target);
             factory_aux->setOptlevel(opt_level);
@@ -688,11 +697,11 @@ EXPORT llvm_dsp_factory* createDSPFactoryFromSignals(const std::string& name_app
     }
 }
         
-EXPORT llvm_dsp_factory* createDSPFactoryFromBoxes(const std::string& name_app, Tree box,
-                                                   int argc, const char* argv[],
-                                                   const std::string& target,
-                                                   std::string& error_msg,
-                                                   int opt_level)
+LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromBoxes(const string& name_app, Tree box,
+                                                      int argc, const char* argv[],
+                                                      const string& target,
+                                                      string& error_msg,
+                                                      int opt_level)
 {
     LOCK_API
     try {
@@ -704,23 +713,23 @@ EXPORT llvm_dsp_factory* createDSPFactoryFromBoxes(const std::string& name_app,
     }
 }
 
-EXPORT llvm_dsp_factory* readDSPFactoryFromBitcode(const string& bit_code, const string& target, string& error_msg,
-                                                   int opt_level)
+LIBFAUST_API llvm_dsp_factory* readDSPFactoryFromBitcode(const string& bit_code, const string& target, string& error_msg,
+                                                      int opt_level)
 {
     LOCK_API
     return readDSPFactoryFromBitcodeAux(MEMORY_BUFFER_CREATE(StringRef(base64_decode(bit_code))), target, error_msg,
                                         opt_level);
 }
 
-EXPORT string writeDSPFactoryToBitcode(llvm_dsp_factory* factory)
+LIBFAUST_API string writeDSPFactoryToBitcode(llvm_dsp_factory* factory)
 {
     LOCK_API
     return (factory) ? factory->writeDSPFactoryToBitcode() : "";
 }
 
 // Bitcode <==> file
-EXPORT llvm_dsp_factory* readDSPFactoryFromBitcodeFile(const string& bit_code_path, const string& target,
-                                                       string& error_msg, int opt_level)
+LIBFAUST_API llvm_dsp_factory* readDSPFactoryFromBitcodeFile(const string& bit_code_path, const string& target,
+                                                          string& error_msg, int opt_level)
 {
     LOCK_API
     ErrorOr<OwningPtr<MemoryBuffer>> buffer = MemoryBuffer::getFileOrSTDIN(bit_code_path);
@@ -732,28 +741,28 @@ EXPORT llvm_dsp_factory* readDSPFactoryFromBitcodeFile(const string& bit_code_pa
     }
 }
 
-EXPORT bool writeDSPFactoryToBitcodeFile(llvm_dsp_factory* factory, const string& bit_code_path)
+LIBFAUST_API bool writeDSPFactoryToBitcodeFile(llvm_dsp_factory* factory, const string& bit_code_path)
 {
     LOCK_API
     return (factory) ? factory->writeDSPFactoryToBitcodeFile(bit_code_path) : false;
 }
 
-EXPORT llvm_dsp_factory* readDSPFactoryFromIR(const string& ir_code, const string& target, string& error_msg,
+LIBFAUST_API llvm_dsp_factory* readDSPFactoryFromIR(const string& ir_code, const string& target, string& error_msg,
                                               int opt_level)
 {
     LOCK_API
     return readDSPFactoryFromIRAux(MEMORY_BUFFER_CREATE(StringRef(ir_code)), target, error_msg, opt_level);
 }
 
-EXPORT string writeDSPFactoryToIR(llvm_dsp_factory* factory)
+LIBFAUST_API string writeDSPFactoryToIR(llvm_dsp_factory* factory)
 {
     LOCK_API
     return (factory) ? factory->writeDSPFactoryToIR() : "";
 }
 
 // IR <==> file
-EXPORT llvm_dsp_factory* readDSPFactoryFromIRFile(const string& ir_code_path, const string& target, string& error_msg,
-                                                  int opt_level)
+LIBFAUST_API llvm_dsp_factory* readDSPFactoryFromIRFile(const string& ir_code_path, const string& target, string& error_msg,
+                                                     int opt_level)
 {
     LOCK_API
     ErrorOr<OwningPtr<MemoryBuffer>> buffer = MemoryBuffer::getFileOrSTDIN(ir_code_path);
@@ -765,7 +774,7 @@ EXPORT llvm_dsp_factory* readDSPFactoryFromIRFile(const string& ir_code_path, co
     }
 }
 
-EXPORT bool writeDSPFactoryToIRFile(llvm_dsp_factory* factory, const string& ir_code_path)
+LIBFAUST_API bool writeDSPFactoryToIRFile(llvm_dsp_factory* factory, const string& ir_code_path)
 {
     LOCK_API
     return (factory) ? factory->writeDSPFactoryToIRFile(ir_code_path) : false;
@@ -777,8 +786,8 @@ EXPORT bool writeDSPFactoryToIRFile(llvm_dsp_factory* factory, const string& ir_
 extern "C" {
 #endif
 
-EXPORT llvm_dsp_factory* createCDSPFactoryFromFile(const char* filename, int argc, const char* argv[],
-                                                   const char* target, char* error_msg, int opt_level)
+LIBFAUST_API llvm_dsp_factory* createCDSPFactoryFromFile(const char* filename, int argc, const char* argv[],
+                                                      const char* target, char* error_msg, int opt_level)
 {
     string error_msg_aux;
     llvm_dsp_factory* factory = createDSPFactoryFromFile(filename, argc, argv, target, error_msg_aux, opt_level);
@@ -786,9 +795,9 @@ EXPORT llvm_dsp_factory* createCDSPFactoryFromFile(const char* filename, int arg
     return factory;
 }
 
-EXPORT llvm_dsp_factory* createCDSPFactoryFromString(const char* name_app, const char* dsp_content, int argc,
-                                                     const char* argv[], const char* target, char* error_msg,
-                                                     int opt_level)
+LIBFAUST_API llvm_dsp_factory* createCDSPFactoryFromString(const char* name_app, const char* dsp_content, int argc,
+                                                        const char* argv[], const char* target, char* error_msg,
+                                                        int opt_level)
 {
     string error_msg_aux;
     llvm_dsp_factory* factory =
@@ -797,11 +806,11 @@ EXPORT llvm_dsp_factory* createCDSPFactoryFromString(const char* name_app, const
     return factory;
 }
     
-EXPORT llvm_dsp_factory* createCDSPFactoryFromSignals(const char* name_app, Signal* signals_aux,
-                                                      int argc, const char* argv[],
-                                                      const char* target,
-                                                      char* error_msg,
-                                                      int opt_level)
+LIBFAUST_API llvm_dsp_factory* createCDSPFactoryFromSignals(const char* name_app, Signal* signals_aux,
+                                                         int argc, const char* argv[],
+                                                         const char* target,
+                                                         char* error_msg,
+                                                         int opt_level)
 {
     string error_msg_aux;
     tvec signals;
@@ -813,12 +822,12 @@ EXPORT llvm_dsp_factory* createCDSPFactoryFromSignals(const char* name_app, Sign
     return factory;
 }
 
-EXPORT llvm_dsp_factory* createCDSPFactoryFromBoxes(const char* name_app,
-                                                    Tree box,
-                                                    int argc, const char* argv[],
-                                                    const char* target,
-                                                    char* error_msg,
-                                                    int opt_level)
+LIBFAUST_API llvm_dsp_factory* createCDSPFactoryFromBoxes(const char* name_app,
+                                                       Tree box,
+                                                       int argc, const char* argv[],
+                                                       const char* target,
+                                                       char* error_msg,
+                                                       int opt_level)
 {
     string error_msg_aux;
     llvm_dsp_factory* factory =
@@ -827,8 +836,8 @@ EXPORT llvm_dsp_factory* createCDSPFactoryFromBoxes(const char* name_app,
     return factory;
 }
 
-EXPORT llvm_dsp_factory* readCDSPFactoryFromBitcode(const char* bit_code, const char* target, char* error_msg,
-                                                    int opt_level)
+LIBFAUST_API llvm_dsp_factory* readCDSPFactoryFromBitcode(const char* bit_code, const char* target, char* error_msg,
+                                                       int opt_level)
 {
     string error_msg_aux;
     llvm_dsp_factory* factory = readDSPFactoryFromBitcode(bit_code, target, error_msg_aux, opt_level);
@@ -836,13 +845,13 @@ EXPORT llvm_dsp_factory* readCDSPFactoryFromBitcode(const char* bit_code, const
     return factory;
 }
 
-EXPORT char* writeCDSPFactoryToBitcode(llvm_dsp_factory* factory)
+LIBFAUST_API char* writeCDSPFactoryToBitcode(llvm_dsp_factory* factory)
 {
     return (factory) ? strdup(writeDSPFactoryToBitcode(factory).c_str()) : nullptr;
 }
 
-EXPORT llvm_dsp_factory* readCDSPFactoryFromBitcodeFile(const char* bit_code_path, const char* target, char* error_msg,
-                                                        int opt_level)
+LIBFAUST_API llvm_dsp_factory* readCDSPFactoryFromBitcodeFile(const char* bit_code_path, const char* target, char* error_msg,
+                                                           int opt_level)
 {
     string error_msg_aux;
     llvm_dsp_factory* factory = readDSPFactoryFromBitcodeFile(bit_code_path, target, error_msg_aux, opt_level);
@@ -850,12 +859,12 @@ EXPORT llvm_dsp_factory* readCDSPFactoryFromBitcodeFile(const char* bit_code_pat
     return factory;
 }
 
-EXPORT bool writeCDSPFactoryToBitcodeFile(llvm_dsp_factory* factory, const char* bit_code_path)
+LIBFAUST_API bool writeCDSPFactoryToBitcodeFile(llvm_dsp_factory* factory, const char* bit_code_path)
 {
     return writeDSPFactoryToBitcodeFile(factory, bit_code_path);
 }
 
-EXPORT llvm_dsp_factory* readCDSPFactoryFromIR(const char* ir_code, const char* target, char* error_msg, int opt_level)
+LIBFAUST_API llvm_dsp_factory* readCDSPFactoryFromIR(const char* ir_code, const char* target, char* error_msg, int opt_level)
 {
     string error_msg_aux;
     llvm_dsp_factory* factory = readDSPFactoryFromIR(ir_code, target, error_msg_aux, opt_level);
@@ -863,12 +872,12 @@ EXPORT llvm_dsp_factory* readCDSPFactoryFromIR(const char* ir_code, const char*
     return factory;
 }
 
-EXPORT char* writeCDSPFactoryToIR(llvm_dsp_factory* factory)
+LIBFAUST_API char* writeCDSPFactoryToIR(llvm_dsp_factory* factory)
 {
     return (factory) ? strdup(writeDSPFactoryToIR(factory).c_str()) : nullptr;
 }
 
-EXPORT llvm_dsp_factory* readCDSPFactoryFromIRFile(const char* ir_code_path, const char* target, char* error_msg,
+LIBFAUST_API llvm_dsp_factory* readCDSPFactoryFromIRFile(const char* ir_code_path, const char* target, char* error_msg,
                                                    int opt_level)
 {
     string            error_msg_aux;
@@ -877,7 +886,7 @@ EXPORT llvm_dsp_factory* readCDSPFactoryFromIRFile(const char* ir_code_path, con
     return factory;
 }
 
-EXPORT bool writeCDSPFactoryToIRFile(llvm_dsp_factory* factory, const char* ir_code_path)
+LIBFAUST_API bool writeCDSPFactoryToIRFile(llvm_dsp_factory* factory, const char* ir_code_path)
 {
     return writeDSPFactoryToIRFile(factory, ir_code_path);
 }
