@@ -4,16 +4,16 @@
  Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
  ---------------------------------------------------------------------
  This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ it under the terms of the GNU Lesser General Public License as published by
+ the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Lesser General Public License for more details.
 
- You should have received a copy of the GNU General Public License
+ You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -21,12 +21,12 @@
 
 #include <stdlib.h>
 #include <iostream>
+#include <algorithm>
 
 #include "exception.hh"
 #include "global.hh"
 #include "occurences.hh"
 #include "recursivness.hh"
-#include "sigtype.hh"
 #include "sigtyperules.hh"
 
 using namespace std;
@@ -40,7 +40,7 @@ static int xVariability(int v, int r)
     // faustassert(v < 3);				// kKonst=0, kBlock=1, kSamp=2
     // faustassert(r==0 | v==2);
     if (r > 1) r = 1;
-    return (int)min(3, v + r);
+    return std::min<int>(3, v + r);
 }
 
 //-------------------------------------------------
@@ -179,7 +179,7 @@ void OccMarkup::setOcc(Tree t, Occurences* occ)
  * @param t signal we want to know the position
  * @return the position in the recursive environment
  */
-static int position (Tree env, Tree t, int p)
+static int position(Tree env, Tree t, int p)
 {
     if (isNil(env)) return 0;	// was not in the environment
     if (hd(env) == t) return p;
