@@ -4,16 +4,16 @@
     Copyright (C) 2003-2014 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -37,10 +37,10 @@
 using namespace emscripten;
 #endif
 
-wasm_dsp_factory* wasm_dynamic_dsp_factory::createWasmDSPFactoryFromString2(const string&         name_app,
-                                                                            const string&         dsp_content,
-                                                                            const vector<string>& argv,
-                                                                            bool                  internal_memory)
+wasm_dsp_factory* wasm_dynamic_dsp_factory::createWasmDSPFactoryFromString2(const string& name_app,
+                                                                        const string& dsp_content,
+                                                                        const vector<string>& argv,
+                                                                        bool internal_memory)
 {
     int         argc1 = 0;
     const char* argv1[64];
@@ -53,10 +53,10 @@ wasm_dsp_factory* wasm_dynamic_dsp_factory::createWasmDSPFactoryFromString2(cons
                                                                wasm_dsp_factory::gErrorMessage, internal_memory);
 }
 
-std::string wasm_dynamic_dsp_factory::generateWasmFromString2(const std::string&           name_app,
-                                                            const std::string&              dsp_content,
-                                                            const std::vector<std::string>& argv,
-                                                            bool                            internal_memory)
+string wasm_dynamic_dsp_factory::generateWasmFromString2(const string& name_app,
+                                                     const string& dsp_content,
+                                                     const vector<string>& argv,
+                                                     bool internal_memory)
 {
     int         argc1 = 0;
     const char* argv1[64];
@@ -65,7 +65,7 @@ std::string wasm_dynamic_dsp_factory::generateWasmFromString2(const std::string&
     }
     argv1[argc1] = nullptr;  // NULL terminated argv
     
-    return generateWasmFromString(name_app, dsp_content, argc1, argv1,wasm_dsp_factory::gErrorMessage, internal_memory);
+    return generateWasmFromString(name_app, dsp_content, argc1, argv1, wasm_dsp_factory::gErrorMessage, internal_memory);
 }
 
 // C++ API
@@ -86,7 +86,7 @@ LIBFAUST_API wasm_dsp_factory* createWasmDSPFactoryFromFile(const string& filena
 }
 
 LIBFAUST_API wasm_dsp_factory* createWasmDSPFactoryFromString(const string& name_app, const string& dsp_content, int argc,
-                                                        const char* argv[], string& error_msg, bool internal_memory)
+                                                            const char* argv[], string& error_msg, bool internal_memory)
 {
     string expanded_dsp_content, sha_key;
 
@@ -107,7 +107,7 @@ LIBFAUST_API wasm_dsp_factory* createWasmDSPFactoryFromString(const string& name
         }
         argv1[argc1] = nullptr;  // NULL terminated argv
 
-        dsp_factory_base* dsp_factory_aux = createFactory(name_app.c_str(), dsp_content.c_str(), argc1, argv1, error_msg, true);
+        dsp_factory_base* dsp_factory_aux = createFactory(name_app, dsp_content, argc1, argv1, error_msg, true);
         if (dsp_factory_aux) {
             dsp_factory_aux->setName(name_app);
             wasm_dsp_factory* factory = new wasm_dsp_factory(dsp_factory_aux);
@@ -121,9 +121,9 @@ LIBFAUST_API wasm_dsp_factory* createWasmDSPFactoryFromString(const string& name
     }
 }
 
-LIBFAUST_API wasm_dsp_factory* createWasmDSPFactoryFromSignals(const std::string& name_app, tvec signals,
-                                                         int argc, const char* argv[], std::string& error_msg,
-                                                         bool internal_memory)
+LIBFAUST_API wasm_dsp_factory* createWasmDSPFactoryFromSignals(const string& name_app, tvec signals,
+                                                            int argc, const char* argv[], string& error_msg,
+                                                            bool internal_memory)
 {
     int         argc1 = 0;
     const char* argv1[64];
@@ -139,7 +139,7 @@ LIBFAUST_API wasm_dsp_factory* createWasmDSPFactoryFromSignals(const std::string
     }
     argv1[argc1] = nullptr;  // NULL terminated argv
     
-    dsp_factory_base* dsp_factory_aux = createFactory(name_app.c_str(), signals, argc1, argv1, error_msg);
+    dsp_factory_base* dsp_factory_aux = createFactory(name_app, signals, argc1, argv1, error_msg);
     if (dsp_factory_aux) {
         dsp_factory_aux->setName(name_app);
         wasm_dsp_factory* factory = new wasm_dsp_factory(dsp_factory_aux);
@@ -150,7 +150,7 @@ LIBFAUST_API wasm_dsp_factory* createWasmDSPFactoryFromSignals(const std::string
     }
 }
 
-LIBFAUST_API std::string generateWasmFromString(const string& name_app, const string& dsp_content, int argc,
+LIBFAUST_API string generateWasmFromString(const string& name_app, const string& dsp_content, int argc,
                                          const char* argv[], string& error_msg, bool internal_memory)
 {
     int         argc1 = 0;
@@ -166,7 +166,7 @@ LIBFAUST_API std::string generateWasmFromString(const string& name_app, const st
     }
     argv1[argc1] = nullptr;  // NULL terminated argv
     
-    dsp_factory_base* dsp_factory_aux = createFactory(name_app.c_str(), dsp_content.c_str(), argc1, argv1, error_msg, true);
+    dsp_factory_base* dsp_factory_aux = createFactory(name_app, dsp_content, argc1, argv1, error_msg, true);
     return (dsp_factory_aux) ? dsp_factory_aux->getBinaryCode() : "";
 }
 
