@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -71,13 +71,13 @@ SimpleForLoopInst* CodeLoop::generateSimpleScalarLoop(const string& counter)
 
 IteratorForLoopInst* CodeLoop::generateSimpleScalarLoop(const std::vector<string>& iterators)
 {
-    std::vector<NamedAddress*> iterators_value_inst;
-    for (const auto& iterator : iterators) {
-        iterators_value_inst.push_back(InstBuilder::genNamedAddress(iterator, Address::kStack));
+    std::vector<NamedAddress*> iterators1;
+    for (const auto& it : iterators) {
+        iterators1.push_back(InstBuilder::genNamedAddress(it, Address::kStack));
     }
 
     BlockInst* block = generateOneSample();
-    IteratorForLoopInst* loop = InstBuilder::genIteratorForLoopInst(iterators_value_inst, false, block);
+    IteratorForLoopInst* loop = InstBuilder::genIteratorForLoopInst(iterators1, false, block);
 
     BasicCloneVisitor cloner;
     return static_cast<IteratorForLoopInst*>(loop->clone(&cloner));
@@ -160,7 +160,6 @@ bool CodeLoop::isEmpty()
  * returns true is this loop has recursive dependencies
  * and must be included in an enclosing loop
  */
-
 bool CodeLoop::hasRecDependencyIn(Tree S)
 {
     CodeLoop* l = this;
