@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -230,7 +230,7 @@ static bool isIntTree(Tree l, vector<int>& v)
 
     } else {
         stringstream error;
-        error << "ERROR in file " << __FILE__ << ':' << __LINE__ << ", not a valid list of numbers : " << boxpp(l)
+        error << "ERROR : file " << __FILE__ << ':' << __LINE__ << ", not a valid list of numbers : " << boxpp(l)
               << endl;
         throw faustexception(error.str());
     }
@@ -315,7 +315,7 @@ static siglist realPropagate(Tree slotenv, Tree path, Tree box, const siglist& l
         faustassert(lsig.size() == 0);
         if (!searchEnv(box, sig, slotenv)) {
             // test YO : diagrams simplification 
-            // fprintf(stderr, "propagate : internal error (slot undefined)\n");
+            // cerr << "propagate : internal error (slot undefined)\n");
             sig = sigInput(++gGlobal->gDummyInput);
         }
         return makeList(sig);
@@ -344,7 +344,7 @@ static siglist realPropagate(Tree slotenv, Tree path, Tree box, const siglist& l
     }
 
     else if (isBoxPrim2(box, &p2)) {
-        //		printf("prim2 recoit : "); print(lsig); printf("\n");
+        // cerr << "prim2 receive : " << ppsig(lsig) << endl;
         faustassert(lsig.size() == 2);
         if (p2 == &sigEnable) {
             if (gGlobal->gEnableFlag) {
@@ -409,35 +409,17 @@ static siglist realPropagate(Tree slotenv, Tree path, Tree box, const siglist& l
 
     else if (isBoxVSlider(box, label, cur, min, max, step)) {
         faustassert(lsig.size() == 0);
-        Tree slider = sigVSlider(normalizePath(cons(label, path)), cur, min, max, step);
-        // Possibly limit the value in [min..max]
-        if (gGlobal->gRangeUI) {
-            return makeList(sigMax(min, sigMin(max, slider)));
-        } else {
-            return makeList(slider);
-        }
+        return makeList(sigVSlider(normalizePath(cons(label, path)), cur, min, max, step));
     }
 
     else if (isBoxHSlider(box, label, cur, min, max, step)) {
         faustassert(lsig.size() == 0);
-        Tree slider = sigHSlider(normalizePath(cons(label, path)), cur, min, max, step);
-        // Possibly limit the value in [min..max]
-        if (gGlobal->gRangeUI) {
-            return makeList(sigMax(min, sigMin(max, slider)));
-        } else {
-            return makeList(slider);
-        }
+        return makeList(sigHSlider(normalizePath(cons(label, path)), cur, min, max, step));
     }
 
     else if (isBoxNumEntry(box, label, cur, min, max, step)) {
         faustassert(lsig.size() == 0);
-        Tree nentry = sigNumEntry(normalizePath(cons(label, path)), cur, min, max, step);
-        // Possibly limit the value in [min..max]
-        if (gGlobal->gRangeUI) {
-            return makeList(sigMax(min, sigMin(max, nentry)));
-        } else {
-            return makeList(nentry);
-        }
+        return makeList(sigNumEntry(normalizePath(cons(label, path)), cur, min, max, step));
     }
 
     else if (isBoxVBargraph(box, label, min, max)) {
@@ -453,14 +435,14 @@ static siglist realPropagate(Tree slotenv, Tree path, Tree box, const siglist& l
     else if (isBoxSoundfile(box, label, chan)) {
         faustassert(lsig.size() == 2);
         Tree    soundfile = sigSoundfile(normalizePath(cons(label, path)));
-        Tree    part      = sigIntCast(lsig[0]);
+        Tree    part      = lsig[0];
         int     c         = tree2int(chan);
         siglist lsig2(c + 2);
         lsig2[0] = sigSoundfileLength(soundfile, part);
         lsig2[1] = sigSoundfileRate(soundfile, part);
 
         // compute bound limited read index : int(max(0, min(ridx,length-1)))
-        Tree ridx = sigIntCast(sigMax(sigInt(0), sigMin(lsig[1], sigSub(lsig2[0], sigInt(1)))));
+        Tree ridx = sigMax(sigInt(0), sigMin(lsig[1], sigSub(lsig2[0], sigInt(1))));
         for (int i1 = 0; i1 < c; i1++) {
             lsig2[i1 + 2] = sigSoundfileBuffer(soundfile, sigInt(i1), part, ridx);
         }
@@ -574,6 +556,7 @@ static siglist realPropagate(Tree slotenv, Tree path, Tree box, const siglist& l
         vector<int> route;
         siglist     outsigs;
         // cerr << "TRACE propagate into a route " << boxpp(box) << endl;
+        // ins, outs, route are casted to int in realeval
         if (isBoxInt(t1, &ins) && isBoxInt(t2, &outs) && isIntTree(t3, route)) {
             // initialize output signals
             for (int i1 = 0; i1 < outs; i1++) outsigs.push_back(sigInt(0));
@@ -596,15 +579,14 @@ static siglist realPropagate(Tree slotenv, Tree path, Tree box, const siglist& l
 
         } else {
             stringstream error;
-            error << "ERROR in file " << __FILE__ << ':' << __LINE__ << ", invalid route expression : " << boxpp(box)
+            error << "ERROR : file " << __FILE__ << ':' << __LINE__ << ", invalid route expression : " << boxpp(box)
                   << endl;
             throw faustexception(error.str());
         }
     }
-    stringstream error;
-    error << "ERROR in file " << __FILE__ << ':' << __LINE__ << ", unrecognised box expression : " << boxpp(box)
-          << endl;
-    throw faustexception(error.str());
+    cerr << "ERROR : file " << __FILE__ << ':' << __LINE__ << ", unrecognised box expression : " << boxpp(box)
+         << endl;
+    faustassert(false);
 
     return siglist();
 }
