@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -70,9 +70,8 @@ int getRecursivness(Tree sig)
 {
     Tree tr;
     if (!getProperty(sig, gGlobal->RECURSIVNESS, tr)) {
-        stringstream error;
-        error << "ERROR in getRecursivness of " << *sig << endl;
-        throw faustexception(error.str());
+        cerr << "ERROR : getRecursivness of " << *sig << endl;
+        faustassert(false);
     }
     return tree2int(tr);
 }
@@ -137,7 +136,7 @@ static int position(Tree env, Tree t, int p)
  * @return the set of symbols
  */
 
-Tree symlistVisit(Tree sig, set<Tree>& visited)
+static Tree symlistVisit(Tree sig, set<Tree>& visited)
 {
     Tree S;
 
@@ -156,7 +155,7 @@ Tree symlistVisit(Tree sig, set<Tree>& visited)
             return U;
         } else {
             vector<Tree> subsigs;
-            int          n = getSubSignals(sig, subsigs, true);  // il faut visiter aussi les tables
+            int          n = getSubSignals(sig, subsigs, true);  // tables have to be visited also
             Tree         U = gGlobal->nil;
             for (int i = 0; i < n; i++) {
                 U = setUnion(U, symlistVisit(subsigs[i], visited));
