@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -31,6 +31,25 @@
 
 using namespace std;
 
+// Uncomment to activate type inferrence tracing
+//#define TRACE(x) x
+
+#define TRACE(x) \
+    {            \
+        ;        \
+    }
+
+AudioType::AudioType(int n, int v, int c, int vec, int b, interval i, res r)
+    : fNature(n), fVariability(v), fComputability(c), fVectorability(vec), fBoolean(b), fInterval(i), fRes(r), fCode(0)
+{
+    TRACE(cerr << gGlobal->TABBER << "Building audioType : n="
+               << "NR"[n] << ", v="
+               << "KB?S"[v] << ", c="
+               << "CI?E"[c] << ", vec="
+               << "VS?TS"[vec] << ", b="
+               << "N?B"[b] << ", i=" << i << endl);
+}  ///< constructs an abstract audio type
+
 bool SimpleType::isMaximal() const  ///< true when type is maximal (and therefore can't change depending of hypothesis)
 {
     return (fNature == kReal) && (fVariability == kSamp) && (fComputability == kExec);
@@ -82,7 +101,8 @@ ostream& SimpleType::print(ostream& dst) const
  */
 ostream& TableType::print(ostream& dst) const
 {
-    dst << "KB?S"[variability()] << "CI?E"[computability()] << " " << fInterval << ":Table(";
+    dst << "NR"[nature()] << "KB?S"[variability()] << "CI?E"[computability()] << "VS?TS"[vectorability()]
+        << "N?B"[boolean()] << " " << fInterval << ":Table(";
     fContent->print(dst);
     return dst << ')';
 }
@@ -239,7 +259,7 @@ Type checkInt(Type t)
 {
     // check that t is an integer
     SimpleType* st = isSimpleType(t);
-    if (st == 0 || st->nature() > kInt) {
+    if (st == nullptr || st->nature() > kInt) {
         stringstream error;
         error << "ERROR : checkInt failed for type " << t << endl;
         throw faustexception(error.str());
@@ -297,7 +317,7 @@ int checkDelayInterval(Type t)
         return int(i.hi + 0.5);
     } else {
         stringstream error;
-        error << "ERROR : invalid delay parameter range: " << i << ". The range must be between 0 and MAX_INT" << endl;
+        error << "ERROR : invalid delay parameter range: " << i << ". The range must be between 0 and INT_MAX" << endl;
         throw faustexception(error.str());
     }
 }
@@ -337,7 +357,7 @@ Tree codeAudioType(AudioType* t)
         r = codeTupletType(nt);
     } else {
         stringstream error;
-        error << "ERROR in codeAudioType() : invalid pointer " << t << endl;
+        error << "ERROR : codeAudioType(), invalid pointer " << t << endl;
         throw faustexception(error.str());
     }
 
@@ -366,7 +386,8 @@ static Tree codeSimpleType(SimpleType* st)
     return CTree::make(gGlobal->SIMPLETYPE, elems);
 }
 
-AudioType* makeSimpleType(int n, int v, int c, int vec, int b, const interval& i){
+AudioType* makeSimpleType(int n, int v, int c, int vec, int b, const interval& i)
+{
     return makeSimpleType(n, v, c, vec, b, i, gGlobal->RES);
 }
 
@@ -392,7 +413,21 @@ AudioType* makeSimpleType(int n, int v, int c, int vec, int b, const interval& i
  */
 static Tree codeTableType(TableType* tt)
 {
-    return tree(gGlobal->TABLETYPE, codeAudioType(tt->content()));
+    vector<Tree> elems;
+    elems.push_back(tree(tt->nature()));
+    elems.push_back(tree(tt->variability()));
+    elems.push_back(tree(tt->computability()));
+    elems.push_back(tree(tt->vectorability()));
+    elems.push_back(tree(tt->boolean()));
+
+    elems.push_back(tree(tt->getInterval().valid));
+    elems.push_back(tree(tt->getInterval().lo));
+    elems.push_back(tree(tt->getInterval().hi));
+
+    elems.push_back(tree(tt->getRes().valid));
+    elems.push_back(tree(tt->getRes().index));
+
+    return CTree::make(gGlobal->TABLETYPE, elems);
 }
 
 AudioType* makeTableType(const Type& ct)
@@ -422,23 +457,7 @@ AudioType* makeTableType(const Type& ct, int n, int v, int c, int vec, int b, co
         return tt;
     } else {
         gGlobal->gAllocationCount++;
-        tt = new TableType(prototype);
-        gGlobal->gMemoizedTypes->set(code, tt);
-        tt->setCode(code);
-        return tt;
-    }
-}
-
-AudioType* makeTableType(const Type& ct, int n, int v, int c, int vec)
-{
-    TableType  prototype(ct, n, v, c, vec);
-    Tree       code = codeAudioType(&prototype);
-    AudioType* tt;
-    if (gGlobal->gMemoizedTypes->get(code, tt)) {
-        return tt;
-    } else {
-        gGlobal->gAllocationCount++;
-        tt = new TableType(prototype);
+        tt = new TableType(ct, n, v, c, vec, b, i);
         gGlobal->gMemoizedTypes->set(code, tt);
         tt->setCode(code);
         return tt;
@@ -457,7 +476,7 @@ static Tree codeTupletType(TupletType* nt)
     return CTree::make(gGlobal->TUPLETTYPE, elems);
 }
 
-AudioType* makeTupletType(const vector<Type>& vt)
+AudioType* makeTupletType(ConstTypes vt)
 {
     TupletType prototype(vt);
     Tree       code = codeAudioType(&prototype);
@@ -474,7 +493,7 @@ AudioType* makeTupletType(const vector<Type>& vt)
     }
 }
 
-AudioType* makeTupletType(const vector<Type>& vt, int n, int v, int c, int vec, int b, const interval& i)
+AudioType* makeTupletType(ConstTypes vt, int n, int v, int c, int vec, int b, const interval& i)
 {
     TupletType prototype(vt, n, v, c, vec, b, i);
     Tree       code = codeAudioType(&prototype);
