@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -38,9 +38,10 @@
 
 //--------------------------------------------------------------------------
 // prototypes
+//--------------------------------------------------------------------------
 
-static void setSigType(Tree sig, Type t);
-static Type getSigType(Tree sig);
+static void        setSigType(Tree sig, Type t);
+static Type        getSigType(Tree sig);
 static TupletType* initialRecType(Tree t);
 static TupletType* maximalRecType(Tree t);
 
@@ -60,16 +61,98 @@ static Type infereDocWriteTblType(Type size, Type init, Type widx, Type wsig);
 static Type infereDocAccessTblType(Type tbl, Type ridx);
 static Type infereWaveformType(Tree lv, Tree env);
 
-TupletType derefRecCert(Type t);
+/**
+ * convert a constant signal into a double using its bounds (not very safe)
+ * @param sig the signal to be converted
+ * @return the mean of its biggest and smallest value
+ */
+static double constSig2double(Tree sig)
+{
+    Type ty = getSigType(sig);
+    if (ty->variability() != kKonst) {
+        throw faustexception(
+            "ERROR : constSig2double, the parameter must be a constant value"
+            " known at compile time\n");
+    }
+    interval bds = ty->getInterval();
+    if (bds.lo != bds.hi) {
+        throw faustexception(
+            "ERROR : constSig2double, constant value with non-singleton interval, don't know what"
+            " to do, please report");
+    }
+    return bds.lo;
+}
 
-static interval arithmetic(int opcode, const interval& x, const interval& y);
+/**
+ * dereference a Type to AudioType and promote its type to TupletType
+ * if the AudioType is not a TupletType, then fails
+ * @param t the type to promote
+ * @return the *t as a TupletType
+ */
+static ::TupletType derefRecCert(Type t)
+{
+    TupletType* p = isTupletType(t);
+    faustassert(p);
+    return *p;
+}
+
+/**
+ * Compute the resulting interval of an arithmetic operation
+ * @param op code of the operation
+ * @param s1 interval of the left operand
+ * @param s2 interval of the right operand
+ * @return the resulting interval
+ */
+static interval arithmetic(int opcode, const interval& x, const interval& y)
+{
+    switch (opcode) {
+        case kAdd:
+            return x + y;
+        case kSub:
+            return x - y;
+        case kMul:
+            return x * y;
+        case kDiv:
+            return x / y;
+        case kRem:
+            return x % y;
+        case kLsh:
+            return x << y;
+        case kARsh:
+            return x >> y;
+        case kGT:
+            return x > y;
+        case kLT:
+            return x < y;
+        case kGE:
+            return x >= y;
+        case kLE:
+            return x <= y;
+        case kEQ:
+            return x == y;
+        case kNE:
+            return x != y;
+        case kAND:
+            return x & y;
+        case kOR:
+            return x | y;
+        case kXOR:
+            return x ^ y;
+        default:
+            cerr << "ERROR : unrecognized opcode : " << opcode << endl;
+            faustassert(false);
+            return {};
+    }
+
+    return interval();
+}
 
 // Uncomment to activate type inferrence tracing
-// #define TRACE(x) x
+//#define TRACE(x) x
 
 #define TRACE(x) \
     {            \
-          ;      \
+        ;        \
     }
 
 /**
@@ -84,23 +167,23 @@ static interval arithmetic(int opcode, const interval& x, const interval& y);
  * @param vdef definitions of all the recursive signal groups (vector of _lists_)
  * @param vdefSizes number of signals in each recursive signal groups
  * @param vtype types of the recursive signals
- * @param inter if set to false, the interval of the new type is the union of the old one and the computed one, otherwise it is the intersection
+ * @param inter if set to false, the interval of the new type is the union of the old one and the computed one,
+ * otherwise it is the intersection
  */
-
-void updateRecTypes(vector<Tree>& vrec, const vector<Tree>& vdef, const vector<int>& vdefSizes,
-                    vector<Type>& vtype, const bool inter)
+void updateRecTypes(vector<Tree>& vrec, const vector<Tree>& vdef, const vector<int>& vdefSizes, vector<Type>& vtype,
+                    const bool inter)
 {
     Type         newType;
     vector<Type> newTuplet;
-    TupletType newRecType;
-    TupletType oldRecType;
-    interval newI;
-    interval oldI;
+    TupletType   newRecType;
+    TupletType   oldRecType;
+    interval     newI;
+    interval     oldI;
 
     const int n = vdef.size();
 
     CTree::startNewVisit();
-    
+
     // init recursive types
     for (int i = 0; i < n; i++) {
         setSigType(vrec[i], vtype[i]);
@@ -119,8 +202,8 @@ void updateRecTypes(vector<Tree>& vrec, const vector<Tree>& vdef, const vector<i
             newTuplet.push_back(newRecType[j]);
             newI = newRecType[j]->getInterval();
             oldI = oldRecType[j]->getInterval();
-            
-            newI = inter ? intersection(newI, oldI) : reunion(newI, oldI);
+
+            newI         = inter ? intersection(newI, oldI) : reunion(newI, oldI);
             newTuplet[j] = newTuplet[j]->promoteInterval(newI);
         }
         vtype[i] = new TupletType(newTuplet);
@@ -132,34 +215,33 @@ void updateRecTypes(vector<Tree>& vrec, const vector<Tree>& vdef, const vector<i
  * @param sig the signal term tree to annotate
  * @param causality when true check causality issues
  */
-
 void typeAnnotation(Tree sig, bool causality)
 {
     gGlobal->gCausality = causality;
     Tree sl             = symlist(sig);
     int  n              = len(sl);
 
-    int size;
+    int  size;
     bool finished = false;
 
-    vector<Tree> vrec;       ///< array of all the recursive signal groups
-    vector<Tree> vdef;       ///< definitions of all the recursive signal groups (vector of _lists_)
-    vector<int>  vdefSizes;  ///< number of signals for each group
-    vector<Type> vtype;      ///< type of the recursive signals
-    vector<Type> vtypeUp;    ///< an upperbound of the recursive signals type
-    vector<TupletType> vUp;  ///< the unfolded version of the variable above
+    vector<Tree>       vrec;       ///< array of all the recursive signal groups
+    vector<Tree>       vdef;       ///< definitions of all the recursive signal groups (vector of _lists_)
+    vector<int>        vdefSizes;  ///< number of signals for each group
+    vector<Type>       vtype;      ///< type of the recursive signals
+    vector<Type>       vtypeUp;    ///< an upperbound of the recursive signals type
+    vector<TupletType> vUp;        ///< the unfolded version of the variable above
 
     vector<vector<int>> vAgeMin;  ///< age of the minimum of every subsignal of the recursive signal
     vector<vector<int>> vAgeMax;  ///< age of the maximum of every subsignal of the recursive signal
 
     // work variables used in widening loop
-    
-    Type newType;
+
+    Type         newType;
     vector<Type> newTuplet;
-    TupletType newRecType;
-    TupletType oldRecType;
-    interval newI;
-    interval oldI;
+    TupletType   newRecType;
+    TupletType   oldRecType;
+    interval     newI;
+    interval     oldI;
 
     // cerr << "Symlist " << *sl << endl;
     for (Tree l = sl; isList(l); l = tl(l)) {
@@ -191,26 +273,26 @@ void typeAnnotation(Tree sig, bool causality)
 
     // cerr << "compute upper bounds for recursive types" << endl;
 
-    for (int k=0; k < gGlobal->gNarrowingLimit; k++) {
+    for (int k = 0; k < gGlobal->gNarrowingLimit; k++) {
         updateRecTypes(vrec, vdef, vdefSizes, vtypeUp, true);
     }
-    
-    for (auto ty : vtypeUp) {
+
+    for (const auto& ty : vtypeUp) {
         vUp.push_back(derefRecCert(ty));
     }
-    
+
     // cerr << "find an upperbound of the least fixpoint" << endl;
-    
+
     while (!finished) {
         updateRecTypes(vrec, vdef, vdefSizes, vtype, false);
-        
+
         // check finished
         finished = true;
         for (int i = 0; i < n; i++) {
             newTuplet.clear();
             // cerr << i << "-" << *vrec[i] << ":" << *getSigType(vrec[i]) << " => " << *vtype[i] << endl;
             if (vtype[i] != getSigType(vrec[i])) {
-                finished = false;
+                finished   = false;
                 newRecType = derefRecCert(vtype[i]);
                 oldRecType = derefRecCert(getSigType(vrec[i]));
                 for (int j = 0; j < vdefSizes[i]; j++) {
@@ -236,7 +318,7 @@ void typeAnnotation(Tree sig, bool causality)
                             newI.hi = vUp[i][j]->getInterval().hi;
                         }
                     }
-                    
+
                     newTuplet[j] = newTuplet[j]->promoteInterval(newI);
                     TRACE(cerr << gGlobal->TABBER << "widening ended : " << newTuplet[j] << endl;)
                 }
@@ -249,7 +331,10 @@ void typeAnnotation(Tree sig, bool causality)
     TRACE(cerr << "type success : " << endl << "BYE" << endl;)
 }
 
-void annotationStatistics()
+/**
+ * Print annotation statistics.
+ */
+static void annotationStatistics()
 {
     cerr << gGlobal->TABBER << "COUNT INFERENCE  " << gGlobal->gCountInferences << " AT TIME "
          << clock() / CLOCKS_PER_SEC << 's' << endl;
@@ -263,7 +348,6 @@ void annotationStatistics()
  * @param sig the signal we want to know the type
  * @return the type of the signal
  */
-
 ::Type getCertifiedSigType(Tree sig)
 {
     Type ty = getSigType(sig);
@@ -284,7 +368,7 @@ void annotationStatistics()
  */
 static void setSigType(Tree sig, Type t)
 {
-    TRACE(cerr << gGlobal->TABBER << "SET FIX TYPE OF " << *sig << " TO TYPE " << *t << endl;)
+    TRACE(cerr << gGlobal->TABBER << "SET FIX TYPE OF " << ppsig(sig) << " TO TYPE " << *t << endl;)
     sig->setType(t);
 }
 
@@ -295,28 +379,14 @@ static void setSigType(Tree sig, Type t)
 static Type getSigType(Tree sig)
 {
     AudioType* ty = (AudioType*)sig->getType();
-    if (ty == 0) {
-        TRACE(cerr << gGlobal->TABBER << "GET FIX TYPE OF " << *sig << " HAS NO TYPE YET" << endl;)
+    if (ty == nullptr) {
+        TRACE(cerr << gGlobal->TABBER << "GET FIX TYPE OF " << ppsig(sig) << " HAS NO TYPE YET" << endl;)
     } else {
-        TRACE(cerr << gGlobal->TABBER << "GET FIX TYPE OF " << *sig << " IS TYPE " << *ty << endl;)
+        TRACE(cerr << gGlobal->TABBER << "GET FIX TYPE OF " << ppsig(sig) << " IS TYPE " << *ty << endl;)
     }
     return ty;
 }
 
-/**
- * dereference a Type to AudioType and promote its type to TupletType
- * if the AudioType is not a TupletType, then fails
- * @param t the type to promote
- * @return the *t as a TupletType
- */
-
-::TupletType derefRecCert(Type t)
-{
-    TupletType* p = isTupletType(t);
-    faustassert(p);
-    return *p;
-}
-
 /**************************************************************************
 
                         Type Inference System
@@ -338,18 +408,18 @@ static Type getSigType(Tree sig)
  */
 static Type T(Tree term, Tree ignoreenv)
 {
-    TRACE(cerr << ++gGlobal->TABBER << "ENTER T() " << *term << endl;)
+    TRACE(cerr << ++gGlobal->TABBER << "ENTER T() " << ppsig(term) << endl;)
 
     if (term->isAlreadyVisited()) {
         Type ty = getSigType(term);
-        TRACE(cerr << --gGlobal->TABBER << "EXIT 1 T() " << *term << " AS TYPE " << *ty << endl);
+        TRACE(cerr << --gGlobal->TABBER << "EXIT 1 T() " << ppsig(term) << " AS TYPE " << *ty << endl);
         return ty;
 
     } else {
         Type ty = infereSigType(term, ignoreenv);
         setSigType(term, ty);
         term->setVisited();
-        TRACE(cerr << --gGlobal->TABBER << "EXIT 2 T() " << *term << " AS TYPE " << *ty << endl);
+        TRACE(cerr << --gGlobal->TABBER << "EXIT 2 T() " << ppsig(term) << " AS TYPE " << *ty << endl);
         return ty;
     }
 }
@@ -371,7 +441,6 @@ static void CheckPartInterval(Tree s, Type t)
  * @param env the type environment
  * @return the type of sig according to environment env
  */
-
 static Type infereSigType(Tree sig, Tree env)
 {
     int    i;
@@ -420,7 +489,7 @@ static Type infereSigType(Tree sig, Tree env)
     else if (isSigDelay(sig, s1, s2)) {
         Type     t1 = T(s1, env);
         Type     t2 = T(s2, env);
-        interval i1  = t2->getInterval();
+        interval i1 = t2->getInterval();
 
         //        cerr << "for sig fix delay : s1 = "
         //				<< t1 << ':' << ppsig(s1) << ", s2 = "
@@ -455,6 +524,8 @@ static Type infereSigType(Tree sig, Tree env)
             return floatCast(t3);  // division always result in a float even with int arguments
         } else if ((i >= kGT) && (i <= kNE)) {
             return boolCast(t3);  // comparison always result in a boolean int
+        } else if (((i >= kLsh) && (i <= kLRsh)) || ((i >= kAND) && (i <= kXOR))) {
+            return intCast(t3);  // boolean and logical operators always result in an int
         } else {
             return t3;  //  otherwise most general of t1 and t2
         }
@@ -614,7 +685,7 @@ static Type infereSigType(Tree sig, Tree env)
         return T(hd(sig), env) * T(tl(sig), env);
     }
 
-    else if (isSigAssertBounds(sig, min, max, cur)){
+    else if (isSigAssertBounds(sig, min, max, cur)) {
         Type     t1 = T(min, env);
         Type     t2 = T(max, env);
         Type     t3 = T(cur, env);
@@ -632,17 +703,17 @@ static Type infereSigType(Tree sig, Tree env)
     else if (isSigLowest(sig, s1)) {
         interval i1 = T(s1, env)->getInterval();
         return makeSimpleType(kReal, kKonst, kComp, kVect, kNum, interval(i1.lo));
-	// change this part   ^^^^^ once there are interval bounds depending on signal type
+        // change this part   ^^^^^ once there are interval bounds depending on signal type
     }
 
     else if (isSigHighest(sig, s1)) {
         interval i1 = T(s1, env)->getInterval();
         return makeSimpleType(kReal, kKonst, kComp, kVect, kNum, interval(i1.hi));
-	// change this part   ^^^^^ once there are interval bounds depending on signal type
+        // change this part   ^^^^^ once there are interval bounds depending on signal type
     }
 
     // unrecognized signal here
-    throw faustexception("ERROR inferring signal type : unrecognized signal\n");
+    throw faustexception("ERROR : inferring signal type, unrecognized signal\n");
     return 0;
 }
 
@@ -652,9 +723,9 @@ static Type infereSigType(Tree sig, Tree env)
 static Type infereProjType(Type t, int i, int vec)
 {
     TupletType* tt = isTupletType(t);
-    if (tt == 0) {
+    if (tt == nullptr) {
         stringstream error;
-        error << "ERROR inferring projection type, not a tuplet type : " << t << endl;
+        error << "ERROR : inferring projection type, not a tuplet type : " << t << endl;
         throw faustexception(error.str());
     }
     // return (*tt)[i]	->promoteVariability(t->variability())
@@ -674,27 +745,41 @@ static Type infereProjType(Type t, int i, int vec)
 /**
  *	Infere the type of the result of writing into a table
  */
-static Type infereWriteTableType(Type tbl, Type wi, Type wd)
+static Type infereWriteTableType(Type tbl, Type wi, Type ws)
 {
     TableType* tt = isTableType(tbl);
-    if (tt == 0) {
+    if (tt == nullptr) {
         stringstream error;
-        error << "ERROR inferring write table type, wrong table type : " << tbl << endl;
+        error << "ERROR : inferring write table type, wrong table type : " << tbl << endl;
         throw faustexception(error.str());
     }
     SimpleType* st = isSimpleType(wi);
-    if (st == 0 || st->nature() > kInt) {
+    if (st == nullptr) {
         stringstream error;
-        error << "ERROR inferring write table type, wrong write index type : " << wi << endl;
+        error << "ERROR : inferring write table type, wrong write index type : " << wi << endl;
         throw faustexception(error.str());
     }
+    TRACE(cerr << gGlobal->TABBER << "infering write table type : wi type = " << wi << endl);
+    TRACE(cerr << gGlobal->TABBER << "infering write table type : wd type = " << ws << endl);
 
-    int n   = tt->nature();
-    int v   = wi->variability() | wd->variability();
-    int c   = wi->computability() | wd->computability();
-    int vec = wi->vectorability() | wd->vectorability();
+    int      n   = ws->nature();
+    int      b   = ws->boolean();
+    int      v   = wi->variability() | ws->variability();
+    int      c   = wi->computability() | ws->computability();
+    int      vec = wi->vectorability() | ws->vectorability();
+    interval i   = ws->getInterval();
+    // return dst << "NR"[nature()] << "KB?S"[variability()] << "CI?E"[computability()] << "VS?TS"[vectorability()]
+    //            << "N?B"[boolean()] << " " << fInterval;
 
-    return makeTableType(tt->content(), n, v, c, vec);
+    TRACE(cerr << gGlobal->TABBER << "infering write table type : n="
+               << "NR"[n] << ", v="
+               << "KB?S"[v] << ", c="
+               << "CI?E"[c] << ", vec="
+               << "VS?TS"[vec] << ", b="
+               << "N?B"[b] << ", i=" << i << endl);
+    Type tbltype = makeTableType(tt->content(), n, v, c, vec, b, i);
+    TRACE(cerr << gGlobal->TABBER << "infering write table type : result=" << tbltype << endl);
+    return tbltype;
 }
 
 /**
@@ -703,18 +788,22 @@ static Type infereWriteTableType(Type tbl, Type wi, Type wd)
 static Type infereReadTableType(Type tbl, Type ri)
 {
     TableType* tt = isTableType(tbl);
-    if (tt == 0) {
+    if (tt == nullptr) {
         stringstream error;
-        error << "ERROR inferring read table type, wrong table type : " << tbl << endl;
+        error << "ERROR : inferring read table type, no table type : " << tbl << endl;
         throw faustexception(error.str());
     }
     SimpleType* st = isSimpleType(ri);
-    if (st == 0 || st->nature() > kInt) {
+    if (st == nullptr) {
         stringstream error;
-        error << "ERROR inferring read table type, wrong write index type : " << ri << endl;
+        error << "ERROR : inferring read table type, no read index type : " << ri << endl;
         throw faustexception(error.str());
     }
-    Type temp = makeSimpleType(tbl->nature(), ri->variability(), kInit | ri->computability(), ri->vectorability(), tbl->boolean(), tbl->getInterval());
+    // Type temp = makeSimpleType(tbl->nature(), ri->variability(), kInit | ri->computability(), ri->vectorability(),
+    //                            tbl->boolean(), tbl->getInterval());
+    Type temp = makeSimpleType(tbl->nature(), tbl->variability() | ri->variability(),
+                               tbl->computability() | ri->computability(), tbl->vectorability() | ri->vectorability(),
+                               tbl->boolean(), tbl->getInterval());
 
     return temp;
 }
@@ -730,11 +819,11 @@ static Type infereDocWriteTblType(Type size, Type init, Type widx, Type wsig)
 {
     checkKonst(checkInt(checkInit(size)));
 
-    Type temp = init->promoteVariability(kSamp)        // difficult to tell, therefore kSamp to be safe
+    Type temp = init->promoteVariability(kSamp)  // difficult to tell, therefore kSamp to be safe
                     ->promoteComputability(widx->computability() | wsig->computability())
-                    ->promoteVectorability(kScal)      // difficult to tell, therefore kScal to be safe
-                    ->promoteNature(wsig->nature())    // nature of the initial and written signal
-                    ->promoteBoolean(wsig->boolean()); // booleanity of the initial and written signal
+                    ->promoteVectorability(kScal)       // difficult to tell, therefore kScal to be safe
+                    ->promoteNature(wsig->nature())     // nature of the initial and written signal
+                    ->promoteBoolean(wsig->boolean());  // booleanity of the initial and written signal
     return temp;
 }
 
@@ -785,7 +874,7 @@ static Type infereFFType(Tree ff, Tree ls, Tree env)
     // An external primitive can't be computed earlier than at initialization.
     // Its variability depends on the variability of its arguments unless it has no arguments,
     // in which case it is considered as rand(), i.e. the result varies at each call.
-    
+
     if (ffarity(ff) == 0) {
         // case of functions like rand()
         return makeSimpleType(ffrestype(ff), kSamp, kInit, kVect, kNum, interval());
@@ -834,16 +923,11 @@ static Type infereFVarType(Tree type)
  */
 static Type infereWaveformType(Tree wfsig, Tree env)
 {
-    bool   iflag = true;
+    bool   iflag = isInt(wfsig->branch(0)->node());
     int    n     = wfsig->arity();
     double lo, hi;
 
-    if (n == 0) {
-        throw faustexception("ERROR empty waveform\n");
-    }
-
     lo = hi = tree2float(wfsig->branch(0));
-    iflag   = isInt(wfsig->branch(0)->node());
     T(wfsig->branch(0), env);
 
     for (int i = 1; i < n; i++) {
@@ -875,70 +959,3 @@ static Type infereXType(Tree sig, Tree env)
     for (int i = 0; i < sig->arity(); i++) vt.push_back(T(sig->branch(i), env));
     return p->infereSigType(vt);
 }
-
-/**
- * Compute the resulting interval of an arithmetic operation
- * @param op code of the operation
- * @param s1 interval of the left operand
- * @param s2 interval of the right operand
- * @return the resulting interval
- */
-static interval arithmetic(int opcode, const interval& x, const interval& y)
-{
-    switch (opcode) {
-        case kAdd:
-            return x + y;
-        case kSub:
-            return x - y;
-        case kMul:
-            return x * y;
-        case kDiv:
-            return x / y;
-        case kRem:
-            return x % y;
-        case kLsh:
-            return x << y;
-        case kARsh:
-            return x >> y;
-        case kGT:
-            return x > y;
-        case kLT:
-            return x < y;
-        case kGE:
-            return x >= y;
-        case kLE:
-            return x <= y;
-        case kEQ:
-            return x == y;
-        case kNE:
-            return x != y;
-        case kAND:
-            return x & y;
-        case kOR:
-            return x | y;
-        case kXOR:
-            return x ^ y;
-        default:
-            stringstream error;
-            error << "ERROR : unrecognized opcode : " << opcode << endl;
-            throw faustexception(error.str());
-    }
-
-    return interval();
-}
-
-double constSig2double(Tree sig)
-{
-    Type ty = getSigType(sig);
-    if (ty->variability() != kKonst) {
-        throw faustexception("ERROR : constSig2double, the parameter must be a constant value"
-			     " known at compile time\n");
-    }
-    interval bds = ty->getInterval();
-    if (bds.lo != bds.hi) {
-        throw faustexception(
-            "ERROR : constSig2double, constant value with non-singleton interval, don't know what"
-            " to do, please report");
-    }
-    return bds.lo;
-}
