@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -22,14 +22,8 @@
 #include "sigConstantPropagation.hh"
 #include <stdlib.h>
 #include <cstdlib>
-#include <map>
-#include "global.hh"
-#include "ppsig.hh"
-#include "property.hh"
+
 #include "signals.hh"
-#include "sigtyperules.hh"
-#include "tlib.hh"
-#include "tree.hh"
 #include "xtended.hh"
 
 /********************************************************************
@@ -125,12 +119,20 @@ Tree SignalConstantPropagation::transformation(Tree sig)
                 return sigProj(i, r);
             }
         } else {
-            stringstream error;
-            error << "ERROR : internal : " << *sig << endl;
-            throw faustexception(error.str());
+            cerr << "ERROR : SignalConstantPropagation::transformation : " << *sig << endl;
+            faustassert(false);
+            return gGlobal->nil;  // Fake return to silence warnings
         }
 
     } else {
         return SignalIdentity::transformation(sig);
     }
 }
+
+// Public API
+Tree constantPropagation(Tree sig, bool trace)
+{
+    SignalConstantPropagation SK;
+    if (trace) SK.trace(true, "ConstProp");
+    return SK.mapself(sig);
+}
