@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -28,22 +28,18 @@
 
 #ifdef WIN32
 #include <windows.h>
+#define kPSEP '\\'
 #else
 #include <libgen.h>
 #include <unistd.h>
 #define GetCurrentDir getcwd
+#define kPSEP '/'
 #endif
 
 #include "exepath.hh"
 
 using namespace std;
 
-#ifdef WIN32
-#define kPSEP '\\'
-#else
-#define kPSEP '/'
-#endif
-
 //-----------------------------------------------------------------
 // removes the exe name from a path (similar to dirname)
 string exepath::dirup(const string& path)
@@ -98,7 +94,7 @@ string exepath::resolvelink(const string& path)
 string exepath::get(const string& name)
 {
     char  buff[1024];
-    DWORD n = GetModuleFileNameA(0, buff, 1024);
+    DWORD n = GetModuleFileNameA(NULL, buff, 1024);
     return n ? dirup(buff) : "";
 }
 
