@@ -1,14 +1,12 @@
 
 // algorithm is required for min and max
 #include <algorithm>
+
 #include "gui/UI.h"
+#include "dsp/one-sample-dsp.h"
 
 using namespace std;
 
-// the struct below are not defined in UI.h
-struct dsp {
-};
-
 struct Meta {
 	void declare(const char*, const char*) {}
 };
