@@ -69,13 +69,11 @@ inline Box getBufferSize()
     destroyLibContext(); \
 }                        \
     
-static void compile(const string& name, Box box, int argc = 0, const char* argv[] = nullptr)
+static void compile(const string& name_app, Box box, int argc = 0, const char* argv[] = nullptr)
 {
-    string error_msg;
-    dsp_factory_base* factory = createCPPDSPFactoryFromBoxes(name, box, argc, argv, error_msg);
-    if (factory) {
-        factory->write(&cout);
-        delete(factory);
+    string error_msg, source = createSourceFromBoxes(name_app, box, "cpp", argc, argv, error_msg);
+    if (source != "") {
+        cout << source;
     } else {
         cerr << error_msg;
     }
@@ -163,11 +161,12 @@ static void test7()
     createLibContext();
     Box box = boxDelay(boxWire(), boxInt(7));
      
-    compile("test7", box, 3, (const char* []){ "-vec", "-lv", "1" });
+    // Vector compilation
+    const char* argv[] = { "-vec", "-lv", "1" };
+    compile("test7", box, 3, argv);
     destroyLibContext();
 }
 
-
 // process = _ <: @(500) + 0.5, @(3000) * 1.5;
 
 static void test8()
@@ -408,8 +407,9 @@ static void test20()
  */
 
 // Using the LLVM backend.
-static void test21(int argc, char* argv[])
+static void test21(int argc, const char* argv[])
 {
+    cout << "test21\n";
     createLibContext();
     {
         Box sl1 = boxVSlider("h:Oscillator/Freq1", boxReal(300), boxReal(100), boxReal(2000), boxReal(0.01));
@@ -428,7 +428,7 @@ static void test21(int argc, char* argv[])
             audio.init("Test", dsp);
             
             // Create GUI
-            GTKUI gtk_ui = GTKUI((char*)"Organ", &argc, &argv);
+            GTKUI gtk_ui = GTKUI((char*)"Organ", &argc, (char***)&argv);
             dsp->buildUserInterface(&gtk_ui);
             
             // Start real-time processing
@@ -449,8 +449,9 @@ static void test21(int argc, char* argv[])
 }
 
 // Using the Interpreter backend.
-static void test22(int argc, char* argv[])
+static void test22(int argc, const char* argv[])
 {
+    cout << "test22\n";
     createLibContext();
     {
         Box sl1 = boxHSlider("v:Oscillator/Freq1", boxReal(300), boxReal(100), boxReal(2000), boxReal(0.01));
@@ -469,7 +470,7 @@ static void test22(int argc, char* argv[])
             audio.init("Test", dsp);
             
             // Create GUI
-            GTKUI gtk_ui = GTKUI((char*)"Organ", &argc, &argv);
+            GTKUI gtk_ui = GTKUI((char*)"Organ", &argc, (char***)&argv);
             dsp->buildUserInterface(&gtk_ui);
             
             // Start real-time processing
@@ -490,8 +491,9 @@ static void test22(int argc, char* argv[])
 }
 
 // Using the Interpreter backend.
-static void test23(int argc, char* argv[])
+static void test23(int argc, const char* argv[])
 {
+    cout << "test23\n";
     interpreter_dsp_factory* factory = nullptr;
     string error_msg;
     
@@ -502,9 +504,28 @@ static void test23(int argc, char* argv[])
         Box sl2 = boxHSlider("v:Oscillator/Freq2", boxReal(300),
                              boxReal(100), boxReal(2000), boxReal(0.01));
         Box box = boxPar(osc(sl1), osc(sl2));
+    
+        // Print the box
+        cout << "Print the box\n";
+        cout << printBox(box, false);
+    
+        // Print the box in shared mode
+        cout << "Print the box with shared identifiers\n";
+        cout << printBox(box, true);
         
-        // Compile the 'bo'x to 'signals'
+        // Compile the 'box' to 'signals'
         tvec signals = boxesToSignals(box, error_msg);
+    
+        // Print the signals
+        cout << "Print the signals\n";
+        for (size_t i = 0; i < signals.size(); i++) {
+            cout << printSignal(signals[i], false) << endl;
+        }
+        // Print the signals in shared mode
+        cout << "Print the signals in shared mode\n";
+        for (size_t i = 0; i < signals.size(); i++) {
+            cout << printSignal(signals[i], true) << endl;
+        }
         
         // Then compile the 'signals' to a DSP factory
         factory = createInterpreterDSPFactoryFromSignals("FaustDSP",
@@ -523,7 +544,7 @@ static void test23(int argc, char* argv[])
         audio.init("Test", dsp);
         
         // Create GUI
-        GTKUI gtk_ui = GTKUI("Organ", &argc, &argv);
+        GTKUI gtk_ui = GTKUI((char*)"Organ", &argc, (char***)&argv);
         dsp->buildUserInterface(&gtk_ui);
         
         // Start real-time processing
@@ -556,8 +577,9 @@ static void test23(int argc, char* argv[])
  */
 
 // Simple polyphonic DSP.
-static void test24(int argc, char* argv[])
+static void test24(int argc, const char* argv[])
 {
+    cout << "test24\n";
     interpreter_dsp_factory* factory = nullptr;
     string error_msg;
     
@@ -588,7 +610,7 @@ static void test24(int argc, char* argv[])
         audio.init("Organ", dsp);
         
         // Create GUI
-        GTKUI gtk_ui = GTKUI((char*)"Organ", &argc, &argv);
+        GTKUI gtk_ui = GTKUI((char*)"Organ", &argc, (char***)&argv);
         dsp->buildUserInterface(&gtk_ui);
         
         // Create MIDI controller
@@ -613,10 +635,70 @@ static void test24(int argc, char* argv[])
     }
 }
 
+// Compile a complete DSP program to a box expression, then to a source string
+static void test25()
+{
+    cout << "test25\n";
+    vector<const char*> lang = { "c", "cpp", "cmajor", "csharp", "dlang", "interp", "jax", "julia", "rust", "wast" };
+    // Context has to be created/destroyed each time
+    for (const auto& it : lang) {
+        createLibContext();
+        {
+            int inputs = 0;
+            int outputs = 0;
+            string error_msg;
+      
+            // Create the oscillator
+            Box osc = DSPToBoxes("FaustDSP", "import(\"stdfaust.lib\"); process = os.osc(440);", 0, nullptr, &inputs, &outputs, error_msg);
+        
+            // Compile it
+            string source = createSourceFromBoxes("FaustDSP", osc, it, 0, nullptr, error_msg);
+            if (source != "") {
+                cout << source;
+            } else {
+                cerr << error_msg;
+            }
+        }
+        destroyLibContext();
+    }
+}
+
+// Compile a complete DSP program to a box expression, then use the result in another expression
+static void test26()
+{
+    cout << "test26\n";
+    createLibContext();
+    {
+        int inputs = 0;
+        int outputs = 0;
+        string error_msg;
+        
+        // Create the filter without parameter
+        Box filter = DSPToBoxes("FaustDSP", "import(\"stdfaust.lib\"); process = fi.lowpass(5);", 0, nullptr, &inputs, &outputs, error_msg);
+        
+        // Create the filter parameters and connect
+        Box cutoff = boxHSlider("cutoff", boxReal(300), boxReal(100), boxReal(2000), boxReal(0.01));
+        Box cutoffAndInput = boxPar(cutoff, boxWire());
+        Box filteredInput = boxSeq(cutoffAndInput, filter);
+    
+        getBoxType(filteredInput, &inputs, &outputs);
+        std::cout << "getBoxType inputs: " << inputs << " outputs: " << outputs << std::endl;
+    
+        // Compile it
+        string source = createSourceFromBoxes("FaustDSP", filteredInput, "cpp", 0, nullptr, error_msg);
+        if (source != "") {
+            cout << source;
+        } else {
+            cerr << error_msg;
+        }
+    }
+    destroyLibContext();
+}
+
 list<GUI*> GUI::fGuiList;
 ztimedmap GUI::gTimedZoneMap;
 
-int main(int argc, char* argv[])
+int main(int argc, const char* argv[])
 {
     test1();
     test2();
@@ -640,6 +722,11 @@ int main(int argc, char* argv[])
     test18();
     test19();
     test20();
+    // Test 'DSPToBoxes' API
+    test25();
+    
+    // Test 'DSPToBoxes' API
+    test26();
     
     // Test with audio, GUI and LLVM backend
     test21(argc, argv);
@@ -652,7 +739,6 @@ int main(int argc, char* argv[])
     
     // Test with audio, GUI, MIDI and Interp backend
     test24(argc, argv);
-    
     return 0;
 }
 
