@@ -71,13 +71,11 @@ inline Signal getBufferSize()
     destroyLibContext(); \
 }                        \
     
-static void compile(const string& name, tvec signals, int argc = 0, const char* argv[] = nullptr)
+static void compile(const string& name_app, tvec signals, int argc = 0, const char* argv[] = nullptr)
 {
-    string error_msg;
-    dsp_factory_base* factory = createCPPDSPFactoryFromSignals(name, signals, argc, argv, error_msg);
-    if (factory) {
-        factory->write(&cout);
-        delete(factory);
+    string error_msg, source = createSourceFromSignals(name_app, signals, "cpp", argc, argv, error_msg);
+    if (source != "") {
+        cout << source;
     } else {
         cerr << error_msg;
     }
@@ -119,8 +117,8 @@ static void test3()
     (
         tvec signals;
         Signal in1 = sigInput(0);
-        signals.push_back(sigDelay(sigAdd(in1, sigReal(0.5)), sigReal(500)));
-        signals.push_back(sigDelay(sigMul(in1, sigReal(1.5)), sigReal(3000)));
+        signals.push_back(sigDelay(sigAdd(in1, sigReal(0.5)), sigInt(500)));
+        signals.push_back(sigDelay(sigMul(in1, sigReal(1.5)), sigInt(3000)));
          
         compile("test3", signals);
     )
@@ -134,8 +132,8 @@ static void test4()
     (
         tvec signals;
         Signal in1 = sigInput(0);
-        signals.push_back(sigAdd(sigDelay(in1, sigReal(500)), sigReal(0.5)));
-        signals.push_back(sigMul(sigDelay(in1, sigReal(3000)), sigReal(1.5)));
+        signals.push_back(sigAdd(sigDelay(in1, sigInt(500)), sigReal(0.5)));
+        signals.push_back(sigMul(sigDelay(in1, sigInt(3000)), sigReal(1.5)));
 
         compile("test4", signals);
     )
@@ -149,8 +147,8 @@ static void test5()
     (
         tvec signals;
         Signal in1 = sigInput(0);
-        signals.push_back(sigDelay(sigAdd(in1, sigReal(0.5)), sigReal(500)));
-        signals.push_back(sigSin(sigDelay(sigDelay(sigAdd(in1, sigReal(0.5)), sigReal(500)), sigReal(600))));
+        signals.push_back(sigDelay(sigAdd(in1, sigReal(0.5)), sigInt(500)));
+        signals.push_back(sigSin(sigDelay(sigDelay(sigAdd(in1, sigReal(0.5)), sigInt(500)), sigInt(600))));
         
         compile("test5", signals);
     )
@@ -164,11 +162,12 @@ static void test6()
     
     tvec signals;
     Signal in1 = sigInput(0);
-    signals.push_back(sigDelay(sigAdd(in1, sigReal(0.5)), sigReal(500)));
-    signals.push_back(sigDelay(sigMul(in1, sigReal(1.5)), sigReal(3000)));
+    signals.push_back(sigDelay(sigAdd(in1, sigReal(0.5)), sigInt(500)));
+    signals.push_back(sigDelay(sigMul(in1, sigReal(1.5)), sigInt(3000)));
     
     // Vector compilation
-    compile("test6", signals, 4, (const char* []){ "-vec", "-lv", "1" , "-double"});
+    const char* argv[] = { "-vec", "-lv", "1" , "-double" };
+    compile("test6", signals, 4, argv);
 
     destroyLibContext();
 }
@@ -181,8 +180,8 @@ static void test7()
     (
         tvec signals;
         Signal in1 = sigInput(0);
-        signals.push_back(sigDelay(sigAdd(in1, sigReal(0.5)), sigReal(500)));
-        signals.push_back(sigAtan2(sigDelay(sigMul(in1, sigReal(1.5)), sigReal(3000)), sigReal(0.5)));
+        signals.push_back(sigDelay(sigAdd(in1, sigReal(0.5)), sigInt(500)));
+        signals.push_back(sigAtan2(sigDelay(sigMul(in1, sigReal(1.5)), sigInt(3000)), sigReal(0.5)));
 
         compile("test7", signals);
     )
@@ -195,11 +194,17 @@ static void equivalent1()
     COMPILER
     (
          tvec signals;
-         Signal s1 = sigAdd(sigDelay(sigInput(0), sigReal(500)), sigReal(0.5));
+         Signal s1 = sigAdd(sigDelay(sigInput(0), sigInt(500)), sigReal(0.5));
          signals.push_back(s1);
          signals.push_back(s1);
-     
+         
          compile("equivalent1", signals);
+     
+         // Print the signals
+         cout << "\nPrint the signals\n";
+         for (size_t i = 0; i < signals.size(); i++) {
+             cout << printSignal(signals[i], false);
+         }
      )
 }
 
@@ -208,13 +213,57 @@ static void equivalent2()
     COMPILER
     (
          tvec signals;
-         signals.push_back(sigAdd(sigDelay(sigInput(0), sigReal(500)), sigReal(0.5)));
-         signals.push_back(sigAdd(sigDelay(sigInput(0), sigReal(500)), sigReal(0.5)));
+         signals.push_back(sigAdd(sigDelay(sigInput(0), sigInt(500)), sigReal(0.5)));
+         signals.push_back(sigAdd(sigDelay(sigInput(0), sigInt(500)), sigReal(0.5)));
      
          compile("equivalent2", signals);
+     
+         // Print the signals
+         cout << "\nPrint the signals\n";
+         for (size_t i = 0; i < signals.size(); i++) {
+             cout << printSignal(signals[i], false);
+         }
     )
 }
 
+// Signals in normal form
+
+static void normalform()
+{
+    COMPILER
+    (
+        tvec signals;
+        signals.push_back(sigAdd(sigAdd(sigDelay(sigDelay(sigInput(0), sigReal(500)), sigReal(200)), sigReal(0.5)), sigReal(3)));
+        signals.push_back(sigMul(sigMul(sigDelay(sigInput(0), sigInt(500)), sigReal(0.5)), sigReal(4)));
+
+        compile("normalform", signals);
+
+        // Print the signals
+        cout << "\nPrint the signals\n";
+        for (size_t i = 0; i < signals.size(); i++) {
+            cout << printSignal(signals[i], false);
+        }
+
+        cout << "\nPrint the signals in shared form\n";
+        for (size_t i = 0; i < signals.size(); i++) {
+            cout << printSignal(signals[i], true);
+        }
+
+        // Compute normal form
+        tvec nf = simplifyToNormalForm2(signals);
+     
+        cout << "\nPrint the signals in normal form\n";
+        for (size_t i = 0; i < nf.size(); i++) {
+            cout << printSignal(nf[i], false);
+        }
+     
+         cout << "\nPrint the signals in normal form in shared mode\n";
+         for (size_t i = 0; i < nf.size(); i++) {
+             cout << printSignal(nf[i], true);
+         }
+     )
+}
+
 // process = @(+(0.5), 500) * vslider("Vol", 0.5, 0, 1, 0.01);
 
 static void test8()
@@ -224,7 +273,7 @@ static void test8()
         tvec signals;
         Signal in1 = sigInput(0);
         Signal slider = sigVSlider("Vol", sigReal(0.5), sigReal(0.0), sigReal(1.0), sigReal(0.01));
-        signals.push_back(sigMul(slider, sigDelay(sigAdd(in1, sigReal(0.5)), sigReal(500))));
+        signals.push_back(sigMul(slider, sigDelay(sigAdd(in1, sigReal(0.5)), sigInt(500))));
         
         compile("test8", signals);
     )
@@ -490,6 +539,7 @@ static void test21()
 // Using the LLVM backend.
 static void test22(int argc, char* argv[])
 {
+    cout << "test22\n";
     createLibContext();
     {
         tvec signals;
@@ -531,6 +581,7 @@ static void test22(int argc, char* argv[])
 // Using the Interpreter backend.
 static void test23(int argc, char* argv[])
 {
+    cout << "test23\n";
     interpreter_dsp_factory* factory = nullptr;
     string error_msg;
     
@@ -589,6 +640,7 @@ with {
 // Simple polyphonic DSP.
 static void test24(int argc, char* argv[])
 {
+    cout << "test24\n";
     interpreter_dsp_factory* factory = nullptr;
     string error_msg;
     
@@ -661,6 +713,7 @@ int main(int argc, char* argv[])
     test7();
     equivalent1();
     equivalent2();
+    normalform();
     test8();
     test9();
     test10();
